<?PHP

/*
  --------------------------------------------------------------------------
  Obliquity of the Ecliptic Calculator

  This program computes the obliquity and the nutation in obliquity
  for any given date, time and Delta t.

  It uses a cookie to store the interface settings for up to 30
  days, unless the cookie is deleted before then.

  Nutation is computed according to both the older 1980 series and
  the newer 2000B series.

  Author:
  Jay Tanner - 2019

  Current Web Site
  www.NeoProgrammics.com

  Development Language:
  PHP v7.1.8

  License:
  This program source code and any related data and graphics are released
  under the provisions of the Creative Commons Attribution-ShareAlike 4.0
  International license.

  https://creativecommons.org/licenses/by-sa/4.0/legalcode
  --------------------------------------------------------------------------
*/


// -----------------------------------------------
// INITIALIZE OUTPUT BUFFER FOR COOKIES TO PREVENT
// THE "headers already sent" ERROR.  THIS MUST BE
// THE VERY FIRST COMMAND OF THE PROGRAM.

   ob_start();

// -----------------------------------
// Speed of light in meters per second

   define ("CLIGHT", "299792458");

// ------------------------------------------
// JULIAN AND GREGORIAN CALENDAR YEAR LENGTHS

// Length of Julian calendar year in seconds.
// Exactly 365.25 days.
   define ("S_PER_JYR", "31557600");

// Length of Gregorian calendar year in seconds.
// Exactly 365.2425 days.
   define ("S_PER_GYR", "31556952");

// ****************************************************************************



   $out1 = $out2 = "";

// YEAR IN GMT
   $cYear = gmdate("Y");


// -----------------------------------------------
// SET THE PATH TO THE PROGRAM TO BE EXECUTED WHEN
// THE SUBMIT BUTTON IS CLICKED.  IN THIS CASE, IT
// IS THIS PROGRAM ITSELF. IT MODIFIES ITSELF EACH
// TIME IT IS CALLED..

   $RunThisProgram = $_SERVER["SCRIPT_NAME"];

// -----------------------------------------------------
// DEFINE A NAME FOR THE COOKIE AND SET EXPIRATION DATE
// TO 30 DAYS IF NOT USED OR DELETED BEFORE THEN.  THIS
// VALUE IS EXPRESSED IN SECONDS FROM THE TIME THAT THE
// COOKIE IS CREATED.  THE VALUE 30*86400 MEANS 30 DAYS
// AT 86400 SECONDS PER DAY, FROM THE time() NOW.

   $CookieName = "ObliquityOfEclipticSettings";

   $ExpirationDate = time() + 30*86400;

// ---------------------------------------
// SET INTERNAL TITLE OF HTML OUTPUT PAGE.

   $HTMLTitle = "Obliquity of the Ecliptic - PHP Science Labs";

// ------------------------------------------------------------
// SET TITLE TO BE DISPLAYED AT THE TOP OF THE INPUT INTERFACE.

   $InterfaceTitle = "<span style='color:white; font-size:15px; line-height:150%;'>Obliquity of the Ecliptic, Nutation in Obliquity and<br>Latitudes of the Arctic/Antarctic Circles</span>";




// -------------------------------------------------
// CHECK IF OR NOT THE [Compute] BUTTON WAS CLICKED.


   if (!isset($_POST["ComputeButton"]))
{

// ----------------------------------------
// DO THIS IF [Compute] BUTTON WAS CLICKED.
//
// IF A COOKIE EXISTS, THEN READ IT AND
// RESET THE PREVIOUS INTERFACE VALUES.

   if (isset($_COOKIE[$CookieName]))
      {
       $CookieDataString = $_COOKIE[$CookieName];

       list ($y,$m,$d,$hh,$mm,$ss,$dTSignSymbol,$dThh,$dTmm,$dTss,$RBState1,$RBState2,$CalMode)
       = preg_split("[\|]", $CookieDataString);

       $m = sprintf("%02d", $m);
       $d = sprintf("%02d", $d);
       $ISOymd = "$y$m$d";
       $dTSignValue = ($dTSignSymbol != "-") ? 1 : -1;
       $CalendarMode = ($CalMode == 0) ? "Julian" : "Gregorian";
      }


   else


// -------------------------------------------------
// IF INITIAL STARTUP AND NO COOKIE EXISTS, THEN SET
// DEFAULT INTERFACE VALUES TO UNIVERSAL DATE AT 00h
// AND STORE IN COOKIE SET TO EXPIRE IN 365 DAYS.

   {
      $y = gmdate("Y");
      $m = gmdate("m");
      $d = gmdate("d");
      $ISOymd = "$y$m$d";

      $hh = gmdate('h');
      $mm = gmdate('i');
      $ss = gmdate('s');

      $dTSignSymbol = "+";
      $dTSignValue = 1;
        $dThh = "00";
        $dTmm = "00";
        $dTss = "00";

    $RBState1 = "";
    $RBState2 = "CHECKED";
    $CalMode  = 1;
    $CalendarMode = "Gregorian";

    $CookieDataString  = "$y|$m|$d|$hh|$mm|$ss|$dTSignSymbol|$dThh|$dTmm|$dTss|$RBState1|$RBState2|$CalMode";

    setcookie ("$CookieName", $CookieDataString, $ExpirationDate);
   }

}




// =============================================
// IF THE [Compute] BUTTON WAS CLICKED, READ AND
// SAVE THE INTERFACE SETTINGS TO A COOKIE.

   if (isset($_POST['ComputeButton']))
{

//     -----------------------------------------
//     READ AND TIDY CURRENT DATE/TIME SETTINGS.

       $y = trim($_POST['y']);
       $m = trim($_POST['m']);
       $d = trim($_POST['d']);

       $m = sprintf("%02d", $m);
       $d = sprintf("%02d", $d);

       $ISOymd = "$y$m$d";

       if ($ISOymd == '')
          {
           $ISOymd = gmdate('Ymd');
           $hh = gmdate('h');
           $mm = gmdate('i');
           $ss = gmdate('s');
          }


       $hh = $_POST['hh'];  $hh = sprintf("%02d",  $hh);
       $mm = $_POST['mm'];  $mm = sprintf("%02d",  $mm);
       $ss = $_POST['ss'];  $ss = sprintf("%1.0f", $ss);  if ($ss < 10) {$ss = "0$ss";}

       $dTSignSymbol = $_POST['dTSignSymbol'];  $dTSignValue = ($dTSignSymbol == "-") ? -1 : 1;
       $dTSignSymbol = ($dTSignValue == 1) ? '+' : '-';
       $dThh = $_POST['dThh'];  $dThh = sprintf("%02d",  $dThh);
       $dTmm = $_POST['dTmm'];  $dTmm = sprintf("%02d",  $dTmm);
       $dTss = $_POST['dTss'];  $dTss = sprintf("%1.0f", $dTss);  if ($dTss < 10) {$dTss = "0$dTss";}

       if (Days_For_HMS($dTSignValue,$dThh,$dTmm,$dTss) == 0)  {$dTSignSymbol = "+";}

// ===========================
// READ CALENDAR MODE SETTING.
//
// 0=Julian  or  1=Gregorian=Default

   $RBIndex = $_POST['RB'];

   if ($RBIndex == 0)
      {
       $CalMode = 0;
       $RBState2 = "";
       $RBState1 = " CHECKED";
       $CalendarMode = "Julian";
      }

   else

      {
       $CalMode = 1;
       $RBState2 = " CHECKED";
       $RBState1 = "";
       $CalendarMode = "Gregorian";
      }

// ============================================================
// COLLECT INTERFACE VALUES AND SETTINGS IN COOKIE DATA STRING
// WITH EACH DATA ELEMENT DELIMITED BY A BAR | CHARACTER.

   $CookieDataString  = "$y|$m|$d|$hh|$mm|$ss|$dTSignSymbol|$dThh|$dTmm|$dTss|$RBState1|$RBState2|$CalMode";

// ==============================
// SAVE THE SETTINGS IN A COOKIE.

   setcookie ("$CookieName", $CookieDataString, $ExpirationDate);

}






// -----------------------------------------------
// CHECK FOR, BLOCK AND REPORT INVALID DATE INPUT,
// OTHERWISE CONTINUE.

   if (Is_Valid_Date($ISOymd, $CalMode))

  {

// ----------------------------
// CONTINUE HERE IF DATE IS OK.
// OTHERWISE, DO else BLOCK.


// ----------------------------------
// COMPUTE JULIAN DAY NUMBERS FOR 12h
// AND 00h ON THE GIVEN DATE.

   $JD12 = JD_For_ISOymd($ISOymd, $CalMode);

   $JD00 = $JD12 - 0.5;

// CONSTRUCT DATE STRING
   $Mmm = Month_String($m);


// -------------------------------------------------
// COMPUTE THE TIME FRACTIONS OF A DAY CORRESPONDING
// THE UT AND THE DELTA T.

   $UTFrac = Days_For_HMS (1, $hh, $mm, $ss);

   $dTFrac = Days_For_HMS ($dTSignValue, $dThh, $dTmm, $dTss);

// ------------------------------------------------------
// ADD THE TIME FRACTIONS TO THE JD00 VALUE TO OBTAIN THE
// GENERAL JD NUMBER FOR THE DATE, TIME AND DELTA T.

   $w  = bcAdd("$JD00",  "$UTFrac", 16);
   $w  = bcAdd("$w",     "$dTFrac", 16);
   $JD = RTrim(RTrim($w, "0"), ".");
   $JD = RTrim(RTrim(bcAdd($JD,  "0.0000000000005", 12), "0"), ".");

// ----------------------------------------------
// COMPUTE TIME IN JULIAN CENTURIES AND MILLENNIA
// CORRESPONDING TO JD AND RECKONED FROM J2000.0

   $T = bcDiv(bcSub($JD, 2451545, 18), "36525", 16);
   $T = sprintf("%+1.12f", $T);

   $DoWStr = DoW_Str_For_ISOymd ($ISOymd, $CalMode);

   $NegBCReminder = ($y < 0) ? "(Neg Year = BC)" : "";

   $UTFrac = sprintf("%+1.8f", $UTFrac);
   $dTFrac = sprintf("%+1.8f", $dTFrac);


// ----------------------------------------
// COMPUTE TRUE OBLIQUITY IN DEGREES FOR JD

   $EpsMeanDeg  = Epsilon_Mean ($JD);
   $EpsMeanDMS  = DMS_For_Deg (FALSE, $EpsMeanDeg,  2, 3, TRUE);

   $EpsMeanDeg = sprintf("%2.10f", $EpsMeanDeg);

   $EpsTrue1980Deg  = $EpsMeanDeg + Delta_Epsilon($JD);
   $EpsTrue2000BDeg = $EpsMeanDeg + dEps_2000B($JD);

   $EpsTrue1980DMS  = DMS_For_Deg (FALSE, $EpsTrue1980Deg,  2, 3, TRUE);
   $EpsTrue2000BDMS = DMS_For_Deg (FALSE, $EpsTrue2000BDeg, 2, 3, TRUE);

   $EpsTrue1980Deg  = sprintf("%2.10f", $EpsTrue1980Deg);
   $EpsTrue2000BDeg = sprintf("%2.10f", $EpsTrue2000BDeg);

// ----------------------------------------------
// COMPUTE NUTATIONS ACCORDING TO IAU SERIES 1980

   $dPsiDeg = Delta_Psi($JD);
   $dRADeg  = $dPsiDeg * cos(deg2rad($EpsTrue1980Deg));
   $dRASS   = sprintf("%+2.3f", $dRADeg * 3600);
   $dRASec  = sprintf("%+2.4f", $dRADeg * 240);
   $dRADeg  = sprintf("%+2.10f", $dRADeg);


   $dPsiSS  = sprintf("%+2.3f",  $dPsiDeg * 3600.0);
   $dPsiDeg = sprintf("%+2.10f", $dPsiDeg);

   $dEpsDeg = Delta_Epsilon($JD);
   $dEpsSS  = sprintf("%+2.3f",  $dEpsDeg * 3600.0);
   $dEpsDeg = sprintf("%+2.10f", $dEpsDeg);


// -----------------------------------------------
// COMPUTE NUTATIONS ACCORDING TO IAU SERIES 2000B

   $dPsi2000BDeg = dPsi_2000B($JD);
   $dPsi2000BSS  = sprintf("%+2.3f",  $dPsi2000BDeg * 3600.0);
   $dPsi2000BDeg = sprintf("%+2.10f", $dPsi2000BDeg);

   $dRA2000BDeg  = $dPsi2000BDeg * cos(deg2rad($EpsTrue2000BDeg));
   $dRA2000BSS   = sprintf("%+2.3f", $dRA2000BDeg * 3600);
   $dRA2000BSec  = sprintf("%+2.4f", $dRA2000BDeg * 240);
   $dRA2000BDeg  = sprintf("%+2.10f", $dRA2000BDeg);

   $dEps2000BDeg = dEps_2000B($JD);
   $dEps2000BSS  = sprintf("%+2.3f",  $dEps2000BDeg * 3600.0);
   $dEps2000BDeg = sprintf("%+2.10f", $dEps2000BDeg);


// COMPUTE ARCTIC CIRCLE LATITUDES FOR 1980 AND 2000B SERIES
// and the difference between them in degrees, km, and mi.
   $w1 = 90-$EpsTrue1980Deg;  $ACLat1980Deg  = sprintf("%2.10f", $w1);
   $w2 = 90-$EpsTrue2000BDeg; $ACLat2000BDeg = sprintf("%2.10f", $w2);
   $wd = abs($w2 - $w1);

   $wdkm = $wd * 111.132866;
   $wdmi = $wd * 69.054762;

   $wdcm = $wd * 111.132866 * 100000 ;
   $wdin = $wd * 69.054762 * 63360;

   $wddeg = sprintf("%2.10f", abs($w2 - $w1));
   $wdcm  = sprintf("%2.3f", abs($wdcm));
   $wdin  = sprintf("%2.3f", abs($wdin));



// mean radius = R = 3956.5464 mi
// mean circum = C = 2 * pi * R = 24859.714 mi
// mean miles per degree =  C / 360 = 69.054762 mi/deg = 111.132866 km/deg



   $ACLat1980DMS  = DMS_for_Deg(FALSE, $ACLat1980Deg,  2, 3, TRUE);
   $ACLat2000BDMS = DMS_for_Deg(FALSE, $ACLat2000BDeg, 2, 3, TRUE);

   $JD = sprintf("%1.8f", $JD);

   $t = sprintf("%+1.10f", $T / 10);
   $T = sprintf("%+1.10f", $T);

// ---------------------------------------
// CONSTRUCT FORMATTED TEXT OUTPUT BLOCK 1

   $out1  =
"

 OBLIQUITY OF THE ECLIPTIC, NUTATION AND LATITUDE OF THE ARCTIC CIRCLE

 NUTATION COMPUTED USING BOTH IAU 1980 SERIES AND 2000B SERIES

 CALENDAR DATE = $y $Mmm $d = $DoWStr ($CalendarMode) $NegBCReminder
 TIME      HMS =  $hh:$mm:$ss = $UTFrac day
 &plusmn; Delta T HMS = $dTSignSymbol$dThh:$dTmm:$dTss = $dTFrac day

 JD NUMBER FOR DATE, TIME AND GIVEN DELTA T
 JD = $JD TT

 TIME VARIABLE CORRESPONDING TO JD
 T = $T = Julian centuries reckoned from J2000.0
 t = $t = Julian millennia reckoned from J2000.0

 ---------------------------------------------------------------------------
 OBLIQUITY OF THE ECLIPTIC

 Eps Mean = $EpsMeanDeg = $EpsMeanDMS (Laskar)
 Eps True = $EpsTrue1980Deg = $EpsTrue1980DMS (IAU 1980 nutation series)
 Eps True = $EpsTrue2000BDeg = $EpsTrue2000BDMS (IAU 2000B nutation series)

 ---------------------------------------------------------------------------
  NUTATION IN OBLIQUITY

  dEps = $dEpsDeg&deg; = $dEpsSS&quot; (IAU 1980 series)
       = $dEps2000BDeg&deg; = $dEps2000BSS&quot; (IAU 2000B series)

 ---------------------------------------------------------------------------
 LATITUDES OF ARCTIC(+) AND ANTARCTIC(&minus;) CIRCLES

 &plusmn; $ACLat1980DMS  =  &plusmn; $ACLat1980Deg&deg; (IAU 1980 Nutation Series)
 &plusmn; $ACLat2000BDMS  =  &plusmn; $ACLat2000BDeg&deg; (IAU 2000B Nutation Series)

 Surface distance difference between nutation series:
 = $wddeg&deg;  =  $wdcm cm  =  $wdin in
";


// ====================================================
// THE MAIN ASTRONOMICAL COMPUTATIONS BLOCK STARTS HERE
// USING THE VALIDATED INTERFACE SETTINGS VALUES.  THE
// OUTPUT STRING WILL BE IN out2





   $out2 = "";



  } // THE MAIN ASTRONOMICAL COMPUTATIONS BLOCK ENDS HERE.


else

// --------------------------------------
// DO THIS IF THE GIVEN DATE WAS INVALID.
//
// REPORT INVALID DATE ERROR IN out2

  {
   $out1 = " ERROR: Invalid date on $CalendarMode calendar.";
   $out2 = "";
  }




$Out1TextRows = 1 + substr_count($out1, "\n");





// ===========================================================
// THE DYNAMIC HTML OUTPUT DOCUMENT FOLLOWS BELOW AND CONTAINS
// THE EMBEDDED VARIABLES DEFINED AND COMPUTED ABOVE.  THESE
// EMBEDDED VARIABLES ARE IDENTIFIED BY THE DOLLAR SIGN SYMBOL
// $ THAT IS ATTACHED TO THEM.
//
// THE CSS STYLES ARE PUT INSIDE THE DOCUMENT FOR CONVENIENCE,
// BUT CAN BE MOVED TO AN EXTERNAL CSS SHEET IF DESIRED.

print <<< _HTML

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/html4/loose.dtd">

<HTML>
<HEAD>

<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">

<TITLE>$HTMLTitle</TITLE>

<style type='text/css'>
BODY  {color:silver; background-color:black;  font-family:Verdana; font-size:14px}
TABLE {font-size:12px; border: 1px solid black;}
TD    {color:black; background-color:white;  line-height:125%; font-size:11px; padding:6px; text-align:center;}
DIV   {color:black; background-color:white;  line-height:125%; font-family:Verdana; text-align:justify; padding:4px; font-size:14px;}
PRE   {color:black; background-color:white;  font-family:monospace; font-size:125%; font-weight:bold; text-align:left; padding:4px; box-shadow:2px 2px 5px #555555;}

CODE  {font-family:monospace; font-size:110%; font-weight:normal;}

TEXTAREA {color:black; background-color:white; font-family:monospace; font-size:15px; font-weight:bold; border-radius: 8px; border:1px solid black; padding:4px; white-space:pre;}

IMG {box-shadow:2px 2px 5px #555555; border:1px solid #888888; border-radius:8px;}

INPUT[type="text"]::-ms-clear {width:0; height:0;}
INPUT[type="text"] {background-color:white; box-shadow:2px 2px 3px #666666; font-family:monospace; font-size:16px; font-weight:bold; text-align:center; border:2px solid black;}
INPUT[type="text"]:focus {background-color:white; box-shadow:2px 2px 3px #666666; font-family:monospace; font-size:16px; border:2px solid red; text-align:center; font-weight:bold;}
INPUT[type="submit"]{font-weight:bold;}
INPUT[type="submit"]:hover {box-shadow:2px 2px 5px #555555; font-weight:bold;}
INPUT[type="checkbox"]:checked {box-shadow:2px 2px 3px #555555;}

HR {background-color:black; height:2px; border:0px;}

A:link,  A:visited {background-color:transparent; color:blue; font-family:Verdana; font-size:90%; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}
A:hover, A:active  {background-color:transparent; color:red;  font-family:Verdana; font-size:90%; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

::selection      {background-color:yellow; color:black;}  /* Only affects IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only affects FireFox and Mozilla */
</style>

</HEAD>

<body>


<table width="686" border="0" align="bottom" bgcolor="black" cellpadding="8" cellspacing="1">

<tr><td style='background-color:black;'>
<form name="InputForm1" method="post" action="$RunThisProgram">

<table width="686" border="0" align="center" bgcolor="black" cellpadding="8" cellspacing="1" style='border:1px solid black;'>

<tr><td style="color:white; background-color:#0000DD; border:2px solid white; border-radius:8px 8px 0px 0px;" colspan="3" align="center" style='border:1px solid white;'><B>$InterfaceTitle</B></TD></TR>

<TR>
<TD ALIGN="center"><B>Date: &nbsp; &plusmn;Y&nbsp; M&nbsp; D</b></TD>
<TD ALIGN="center"><B>Time UT: &nbsp; H M S</b></TD>
<TD ALIGN="center"><B>&Delta;T: &nbsp; &plusmn; H M S</b></TD>
</TR>

<TR>
<TD ALIGN="center">
<INPUT NAME="y" TYPE="text" VALUE="$y" SIZE="6" MAXLENGTH="5">
<INPUT NAME="m" TYPE="text" VALUE="$m" SIZE="3" MAXLENGTH="2">
<INPUT NAME="d" TYPE="text" VALUE="$d" SIZE="3" MAXLENGTH="2">
</TD>

<TD ALIGN="center">
<INPUT NAME="hh" TYPE="text" VALUE="$hh" SIZE="3" MAXLENGTH="2">
<INPUT NAME="mm" TYPE="text" VALUE="$mm" SIZE="3" MAXLENGTH="2">
<INPUT NAME="ss" TYPE="text" VALUE="$ss" SIZE="3" MAXLENGTH="2">
</TD>

<TD ALIGN="center">
<INPUT NAME="dTSignSymbol" TYPE="text" VALUE="$dTSignSymbol" SIZE="1" MAXLENGTH="1">
<INPUT NAME="dThh"   TYPE="text" VALUE="$dThh"   SIZE="3" MAXLENGTH="2">
<INPUT NAME="dTmm"   TYPE="text" VALUE="$dTmm"   SIZE="3" MAXLENGTH="2">
<INPUT NAME="dTss"   TYPE="text" VALUE="$dTss"   SIZE="3" MAXLENGTH="2">
</TD>
</TR>

<TR>
<TD COLSPAN="3" ALIGN="center" BGCOLOR="GreenYellow"><B>Calendar Mode</B>: &nbsp;
<INPUT NAME="RB" TYPE="radio" VALUE="0"$RBState1>Julian
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
<INPUT NAME="RB" TYPE="radio" VALUE="1"$RBState2>Gregorian
</TD>
</TR>

<TR><TD COLSPAN="3" ALIGN="center" BGCOLOR="black"><INPUT TYPE="submit" NAME="ComputeButton" VALUE="Compute"></TD></TR>

</TABLE>
</FORM></td></tr>

<TR><TD ALIGN="left" style='background-color:black;'>
<B><TEXTAREA CLASS="BgWhite" NAME="PHPCode1" COLS="82" ROWS="$Out1TextRows" READONLY="readonly" style='font-weight:bold;'>$out1</TEXTAREA></B>
</TD></TR>

<TR><TD ALIGN="left" BGCOLOR="white" style='border-radius:8px;'>
<DIV CLASS="BgWhite" style="text-align:justify; padding:6px;">
<div style='text-align:center;'><img src='images/obliquity_earth_orbital_inclination.png' alt=''></div>
<br><br>
The obliquity of the ecliptic is essentially the angle of inclination of the Earth&#39;s equator with respect to the plane of its orbit, which to us, produces the apparent relative tilt of the Earth&#39;s polar axis which is the principal cause of the annual seasons.
<br><br>
This angle is approximately &plusmn;23.45 degrees and is the apparent maximum/minimum angle of the sun above or below the equator during the year as viewed from Earth. &nbsp; This value varies very slowly as the orbit of the Earth evolves over time in response to various internal and external influences, such as uneven mass distribution, tidal and gravitational perturbations of the sun, moon and other bodies, etc.
<br><br>
This is one of the fundamental values we need to know when computing the apparent position of a planet as viewed from the Earth at any given moment as it defines the instantaneous inclination of the Earth&#39;s orbit.&nbsp;

<br><br>
The ecliptic obliquity can also be used to determine the latitudes of the arctic and antarctic circles, however these values can vary slightly depending on the computational methods, the values of the applied constants, time scale, coordinates system, the applied geoid model, the accuracy of the applied nutational theory, among other factors, some of which are unpredictable.
<br><br>

In the computation of the arctic/antarctic circles, the difference between applying the IAU 1980 nutation series or the 2000B series only amounts to a matter of inches difference on the ground.&nbsp;  Due to the margin of error involved in such computations and the degree to which these margins overlap, this small difference is generally too insignificant to matter when computing the latitudes of the arctic/antarctic circles.
<br><br>
Laskar&#39;s formula for the mean obliquity theoretically spans 10,000 years, however, I have yet to find any reliable information as to the full validity span of either of the nutational series, so how accurately the computed nutations would apply to the mean obliquity 6000 years ago or in the future from now is still a bit hazy.

<br><br>
Over a period of several years, these values have varied as measurement-technologies improved their accuracy and consistency.  They will continue to vary for the same reasons in the future, so the same predictions based on them will also vary to a small degree from generation to generation.
<br><br>
As for the theoretical accuracy, below are some excerpts taken from the original paper summary:

<pre style='background-color:LightYellow; line-height:125%; font-size:15px; border:1px solid black; border-radius:8px;'>
..... Excerpts from J. Laskar&#39;s original paper summary:

Using the methods of the general theory given in (Laskar, 1985),
we have analytically computed the differential system giving the
secular variations of the orbital elements for the 8 major planets,
at the order 2 with respect to the masses and up to degree 5 in the
eccentricity-inclination variables with a relative precision of 10E-6.

Relativistic and lunar perturbations are included.

The entire system is integrated numerically over 10,000 years and then
developed in Taylor expansion around J2000.

We obtain new polynomial secular terms for the inner planets up to the
power 10 of the time.

Comparisons are made with Bretagon&#39;s theory VSOP82 and with the
numerically integrated JPL ephemeris, DE102 (Newhall et al., 1983).

The global accuracy is approximately 0.04202&#34; / 1,000 yr for the
inclination of the Earth.

Using the theory of the rotation of the rigid Earth of Kinoshita (1977),
we derive new formulas for the precessional quantities, up to the 10th
power of t, and valid over 10,000 years.

.....
</pre>

<ul style='width:500px;'>
<li>Ref: <a href="http://en.wikipedia.org/wiki/Ecliptic" target='_blank'>http://en.wikipedia.org/wiki/Ecliptic</a>

<br><br>
The algorithm used in the above program is based on data published by <i>J. Laskar</i>
in <i>Astronomy and Astrophysics</i>, Vol 157, p68 (1986), <i>New Formulas for the Precession, Valid Over 10000 years, Table 8</i>.
<br><br>
<img src="images/A&A_Vol_157-Page-68ex.png" alt=" Excerpt From Paper ">
<br><br>
Provided by the <b>NASA Astrophysics Data System</b>
<BR><BR>
<HR>
</li>

<li>
<B>J. Laskar&#39;s Formula For The Mean Obliquity</B>
<br><br>
In this case, the time variable (<B>t</B>) of <i>Table 8</i> is reckoned in terms of
decamillennia (10,000 Julian years) from J2000.0 corresponding to
the JD for any given moment and may be found from:
<br><br>

<img src='images/t_10000.png'>

<br><br>
The value <b>84381.448</b> seconds of arc equates to <b>23&deg; 26&#39; 21.448&quot;</b>
which is the mean obliquity for the ecliptic of <b>J2000.0</b> TT, where:<br>
<b>J2000.0 = 2000 Jan 01 at 12:00:00 TT</b>
<br>
<br><br>
</li>


<li>
Based on the <i>Table 8</i> data above (NGT column), the
mean obliquity (in&nbsp;arc&nbsp;seconds) of the ecliptic at time (<b>t</b>) is:
<br><br>

<img src='images/epsilon.png'>

<br><br>
This value of the obliquity (&epsilon;) is the mean value. For the true, apparent obliquity of the date, a small correction for the nutation in obliquity must be applied.
<br><br>

</li>
</ul>
<DIV style="font-size:10px; text-align:center"><B>Jay Tanner - $cYear</B></DIV>

</td></tr>

<tr><td colspan='1' style='background-color:black;'>&nbsp;</td></tr>
<tr><td style='color:black; background-color:white; font-size:12px; text-align:center; border-radius:8px;' colspan='2'>
J. Laskar Paper - <a href='laskar_paper/' target='_blank' >Read The Original J. Laskar Paper on Long-Term Obliquity</a>
</td></tr>



</table>

<br>

<HR>



</BODY>
</HTML>

_HTML;



/*

   ************************************************
   THIS FUNCTION COMPUTES THE NUTATION IN LONGITUDE
   ACCORDING TO THE IAU 2000B NUTATION SERIES.  THE
   RETURNED RESULT IS IN DECIMAL DEGREES.

   THIS SERIES CONSISTS OF 77 TERMS.

*/

   function dPsi_2000B ($JD)
{

// ------------------------------------------------------
// Compute time in Julian centuries reckoned from J2000.0
// correspomding to the JD number.

   $T = ($JD - 2451545.0) / 36525.0;

// -------------------
// Compute powers of T

   $T2 = $T * $T;
   $T3 = $T * $T2;
   $T4 = $T * $T3;

// -----------------------------------
// Mean anomaly of the Moon in radians

$L  = deg2rad((485868.249036 + 1717915923.2178*$T + 31.8792*$T2
    + 0.051635*$T3 - 0.0002447*$T4) / 3600.0);


// ----------------------------------
// Mean anomaly of the Sun in radians

$Lp = deg2rad((1287104.79305 + 129596581.0481*$T
    - 0.5532*$T2  + 0.000136*$T3 - 0.00001149*$T4) / 3600.0);


// ----------------------------------------------------
// Mean argument of the latitude of the Moon in radians

$F  = deg2rad((335779.526232 + 1739527262.8478*$T
    - 12.7512*$T2 - 0.001037*$T3 + 0.00000417*$T4) / 3600.0);


// ---------------------------------------------------
// Mean elongation of the Moon from the Sun in radians

$D  = deg2rad((1072260.70369 + 1602961601.2090*$T
    - 6.3706*$T2  + 0.006593*$T3 - 0.00003169*$T4) / 3600.0);


// ----------------------------------------------------------------
// Mean longitude of the mean ascending node of the Moon in radians

$Om = deg2rad((450160.398036 - 6962890.5431*$T
    + 7.4722*$T2  + 0.007702*$T3 - 0.00005939*$T4) / 3600.0);


// SUM SERIES FOR NUTATION IN LONGITUDE (dPsi) IN ARC SEC * 10E+7

$s = 0;
$s += (-172064161 - 174666*$T)*sin($Om) + 33386*cos($Om);
$s += (-13170906 - 1675*$T)*sin(2*($F - $D + $Om)) - 13696*cos(2*($F - $D + $Om));
$s += (-2276413 - 234*$T)*sin(2*($F + $Om)) + 2796*cos(2*($F + $Om));
$s += (2074554 + 207*$T)*sin(2*$Om) - 698*cos(2*$Om);
$s += (1475877 - 3633*$T)*sin($Lp) + 11817*cos($Lp);
$s += (-516821 + 1226*$T)*sin($Lp + 2*($F - $D + $Om)) - 524*cos($Lp + 2*($F - $D + $Om));
$s += (711159 + 73*$T)*sin($L) - 872*cos($L);
$s += (-387298 - 367*$T)*sin(2*$F + $Om) + 380*cos(2*$F + $Om);
$s += (-301461 - 36*$T)*sin($L + 2*($F + $Om)) + 816*cos($L + 2*($F + $Om));
$s += (215829 - 494*$T)*sin(2*($F - $D + $Om) - $Lp) + 111*cos(2*($F - $D + $Om) - $Lp);
$s += (128227 + 137*$T)*sin(2*($F - $D) + $Om) + 181*cos(2*($F - $D) + $Om);
$s += (123457 + 11*$T)*sin(2*($F + $Om) - $L) + 19*cos(2*($F + $Om) - $L);
$s += (156994 + 10*$T)*sin(2*$D - $L) - 168*cos(2*$D - $L);
$s += (63110 + 63*$T)*sin($L + $Om) + 27*cos($L + $Om);
$s += (-57976 - 63*$T)*sin($Om - $L) - 189*cos($Om - $L);
$s += (-59641 - 11*$T)*sin(2*($F + $D + $Om) - $L) + 149*cos(2*($F + $D + $Om) - $L);
$s += (-51613 - 42*$T)*sin($L + 2*$F + $Om) + 129*cos($L + 2*$F + $Om);
$s += (45893 + 50*$T)*sin(2*($F - $L) + $Om) + 31*cos(2*($F - $L) + $Om);
$s += (63384 + 11*$T)*sin(2*$D) - 150*cos(2*$D);
$s += (-38571 - $T)*sin(2*($F + $D + $Om)) + 158*cos(2*($F + $D + $Om));
$s += 32481*sin(2*($F - $Lp - $D + $Om));
$s -= 47722*sin(2*($D - $L)) + 18*cos(2*($D - $L));
$s += (-31046 - $T)*sin(2*($L + $F + $Om)) + 131*cos(2*($L + $F + $Om));
$s += 28593*sin($L + 2*($F - $D + $Om)) - cos($L + 2*($F - $D + $Om));
$s += (20441 + 21*$T)*sin(2*$F - $L + $Om) + 10*cos(2*$F - $L + $Om);
$s += 29243*sin(2*$L) - 74*cos(2*$L);
$s += 25887*sin(2*$F) - 66*cos(2*$F);
$s += (-14053 - 25*$T)*sin($Lp + $Om) + 79*cos($Lp + $Om);
$s += (15164 + 10*$T)*sin(2*$D - $L + $Om) + 11*cos(2*$D - $L + $Om);
$s += (-15794 + 72*$T)*sin(2*($Lp + $F - $D + $Om)) - 16*cos(2*($Lp + $F - $D + $Om));
$s += 21783*sin(2*($D - $F)) + 13*cos(2*($D - $F));
$s += (-12873 - 10*$T)*sin($L - 2*$D + $Om) - 37*cos($L - 2*$D + $Om);
$s += (-12654 + 11*$T)*sin($Om - $Lp) + 63*cos($Om - $Lp);
$s -= 10204*sin(2*($F + $D) - $L + $Om) - 25*cos(2*($F + $D) - $L + $Om);
$s += (16707 - 85*$T)*sin(2*$Lp) - 10*cos(2*$Lp);
$s -= 7691*sin($L + 2*($F + $D + $Om)) - 44*cos($L + 2*($F + $D + $Om));
$s -= 11024*sin(-2*$L + 2*$F) + 14*cos(2*($F - $L));
$s += (7566 - 21*$T)*sin($Lp + 2*($F + $Om)) - 11*cos($Lp + 2*($F + $Om));
$s += (-6637 - 11*$T)*sin(2*($F + $D) + $Om) + 25*cos(2*($F + $D) + $Om);
$s += (-7141 + 21*$T)*sin(2*($F + $Om) - $Lp) + 8*cos(2*($F + $Om) - $Lp);
$s += (-6302 - 11*$T)*sin(2*$D + $Om) + 2*cos(2*$D + $Om);
$s += (5800 + 10*$T)*sin($L + 2*($F - $D) + $Om) + 2*cos($L + 2*($F - $D) + $Om);
$s += 6443*sin(2*($L + $F - $D + $Om)) - 7*cos(2*($L + $F - $D + $Om));
$s += (-5774 - 11*$T)*sin(2*($D - $L) + $Om) - 15*cos(2*($D - $L) + $Om);
$s -= 5350*sin(2*($L + $F) + $Om) - 21*cos(2*($L + $F) + $Om);
$s += (-4752 - 11*$T)*sin(2*($F - $D) - $Lp + $Om) - 3*cos(2*($F - $D) - $Lp + $Om);
$s += (-4940 - 11*$T)*sin($Om - 2*$D) - 21*cos($Om - 2*$D);
$s += 7350*sin(2*$D - $L - $Lp) - 8*cos(2*$D - $L - $Lp);
$s += 4065*sin(2*($L - $D) + $Om) + 6*cos(2*($L - $D) + $Om);
$s += 6579*sin($L + 2*$D) - 24*cos($L + 2*$D);
$s += 3579*sin($Lp + 2*($F - $D) + $Om) + 5*cos($Lp + 2*($F - $D) + $Om);
$s += 4725*sin($L - $Lp) - 6*cos($L - $Lp);
$s -= 3075*sin(2*($F - $L  + $Om)) + 2*cos(2*($F - $L  + $Om));
$s -= 2904*sin(3*$L + 2*($F + $Om)) - 15*cos(3*$L + 2*($F + $Om));
$s += 4348*sin(2*$D - $Lp) - 10*cos(2*$D - $Lp);
$s -= 2878*sin($L - $Lp + 2*($F + $Om)) - 8*cos($L - $Lp + 2*($F + $Om));
$s -= 4230*sin($D) - 5*cos($D);
$s -= 2819*sin(2*($F + $D + $Om) - $L - $Lp) - 7*cos(2*($F + $D + $Om) - $L - $Lp);
$s -= 4056*sin(2*$F - $L) - 5*cos(2*$F - $L);
$s -= 2647*sin(2*($F + $D + $Om) - $Lp) - 11*cos(2*($F + $D + $Om) - $Lp);
$s -= 2294*sin($Om - 2*$L) + 10*cos($Om - 2*$L);
$s += 2481*sin($L + $Lp + 2*($F + $Om)) - 7*cos($L + $Lp + 2*($F + $Om));
$s += 2179*sin(2*$L + $Om) - 2*cos(2*$L + $Om);
$s += 3276*sin($Lp - $L + $D) + cos($Lp - $L + $D);
$s -= 3389*sin($L + $Lp) - 5*cos($L + $Lp);
$s += 3339*sin($L + 2*$F) - 13*cos($L + 2*$F);
$s -= 1987*sin(2*($F - $D) - $L + $Om) + 6*cos(2*($F - $D) - $L + $Om);
$s -= 1981*sin($L + 2*$Om);
$s += 4026*sin($D - $L) - 353*cos($D - $L);
$s += 1660*sin($D + 2*($F + $Om)) - 5*cos($D + 2*($F + $Om));
$s -= 1521*sin(2*($F + 2*$D + $Om) - $L) - 9*cos(2*($F + 2*$D + $Om) - $L);
$s += 1314*sin($Lp - $L + $D + $Om);
$s -= 1283*sin(2*($F - $Lp - $D) + $Om);
$s -= 1331*sin($L + 2*($F + $D) + $Om) - 8*cos($L + 2*($F + $D) + $Om);
$s += 1383*sin(2*($F - $L + $D + $Om)) - 2*cos(2*($F - $L + $D + $Om));
$s += 1405*sin(2*$Om - $L) + 4*cos(2*$Om - $L);
$s += 1290*sin($L + $Lp + 2*($F - $D + $Om));

$dPsiDeg = $s / 36000000000;

return $dPsiDeg;

} // End of  dPsi_2000B()


// ****************************
// ****************************


/*

   ************************************************
   THIS FUNCTION COMPUTES THE NUTATION IN OBLIQUITY
   ACCORDING TO THE IAU 2000B NUTATION SERIES.  THE
   RETURNED RESULT IS IN DECIMAL DEGREES.

   THIS SERIES CONSISTS OF 77 TERMS.

*/

   function dEps_2000B ($JD)
{

// ------------------------------------------------------
// Compute time in Julian centuries reckoned from J2000.0
// correspomding to the JD number.

   $T = ($JD - 2451545.0) / 36525.0;

// -------------------
// Compute powers of T

   $T2 = $T * $T;
   $T3 = $T * $T2;
   $T4 = $T * $T3;


// -----------------------------------
// Mean anomaly of the Moon in radians

$L  = deg2rad((485868.249036 + 1717915923.2178*$T + 31.8792*$T2
    + 0.051635*$T3 - 0.00024470*$T4) / 3600.0);


// ----------------------------------
// Mean anomaly of the Sun in radians

$Lp = deg2rad((1287104.79305 + 129596581.0481*$T
    - 0.5532*$T2  + 0.000136*$T3 - 0.00001149*$T4) / 3600.0);


// ----------------------------------------------------
// Mean argument of the latitude of the Moon in radians

$F  = deg2rad((335779.526232 + 1739527262.8478*$T
    - 12.7512*$T2 - 0.001037*$T3 + 0.00000417*$T4) / 3600.0);


// ---------------------------------------------------
// Mean elongation of the Moon from the Sun in radians

$D  = deg2rad((1072260.70369 + 1602961601.2090*$T
    - 6.3706*$T2  + 0.006593*$T3 - 0.00003169*$T4) / 3600.0);


// ----------------------------------------------------------------
// Mean longitude of the mean ascending node of the Moon in radians

$Om = deg2rad((450160.398036 - 6962890.5431*$T
    + 7.4722*$T2  + 0.007702*$T3 - 0.00005939*$T4) / 3600.0);


// SUM SERIES FOR NUTATION IN OBLIQUITY (dEps) IN ARC SEC * 10E+7

$s = 0;
$s += (92052331 + 9086*$T)*cos($Om) + 15377*sin($Om);
$s += (5730336 - 3015*$T)*cos(2*($F - $D + $Om)) - 4587*sin(2*($F - $D + $Om));
$s += (978459 - 485*$T)*cos(2*($F + $Om)) + 1374*sin(2*($F + $Om));
$s += (-897492 + 470*$T)*cos(2*$Om) - 291*sin(2*$Om);
$s += (73871 - 184*$T)*cos($Lp) - 1924*sin($Lp);
$s += (224386 - 677*$T)*cos($Lp + 2*($F - $D + $Om)) - 174*sin($Lp + 2*($F - $D + $Om));
$s -= 6750*cos($L) - 358*sin($L);
$s += (200728 + 18*$T)*cos(2*$F + $Om) + 318*sin(2*$F + $Om);
$s += (129025 - 63*$T)*cos($L + 2*($F + $Om)) + 367*sin($L + 2*($F + $Om));
$s += (-95929 + 299*$T)*cos(2*($F - $D + $Om) - $Lp) + 132*sin(2*($F - $D + $Om) - $Lp);
$s += (-68982 - 9*$T)*cos(2*($F - $D) + $Om) + 39*sin(2*($F - $D) + $Om);
$s += (-53311 + 32*$T)*cos(2*($F + $Om) - $L) - 4*sin(2*($F + $Om) - $L);
$s -= 1235*cos(2*$D - $L) - 82*sin(2*$D - $L);
$s -= 33228*cos($L + $Om) + 9*sin($L + $Om);
$s += 31429*cos($Om - $L) - 75*sin($Om - $L);
$s += (25543 - 11*$T)*cos(2*($F + $D + $Om) - $L) + 66*sin(2*($F + $D + $Om) - $L);
$s += 26366*cos($L + 2*$F + $Om) + 78*sin($L + 2*$F + $Om);
$s += (-24236 - 10*$T)*cos(2*($F - $L) + $Om) + 20*sin(2*($F - $L) + $Om);
$s -= 1220*cos(2*$D) - 29*sin(2*$D);
$s += (16452 - 11*$T)*cos(2*($F + $D + $Om)) + 68*sin(2*($F + $D + $Om));
$s -= 13870*cos(2*($F - $Lp - $D + $Om));
$s += 477*cos(2*($D - $L)) - 25*sin(2*($D - $L));
$s += (13238 - 11*$T)*cos(2*($L + $F + $Om)) + 59*sin(2*($L + $F + $Om));
$s += (-12338 + 10*$T)*cos($L + 2*($F - $D + $Om)) - 3*sin($L + 2*($F - $D + $Om));
$s -= 10758*cos(2*$F - $L + $Om) + 3*sin(2*$F - $L + $Om);
$s -= 609*cos(2*$L) - 13*sin(2*$L);
$s -= 550*cos(2*$F) - 11*sin(2*$F);
$s += (8551 - 2*$T)*cos($Lp + $Om) - 45*sin($Lp + $Om);
$s -= 8001*cos(2*$D - $L + $Om) + sin(2*$D - $L + $Om);
$s += (6850 - 42*$T)*cos(2*($Lp + $F - $D + $Om)) - 5*sin(2*($Lp + $F - $D + $Om));
$s -= 167*cos(2*($D - $F)) - 13*sin(2*($D - $F));
$s += 6953*cos($L - 2*$D + $Om) - 14*sin($L - 2*$D + $Om);
$s += 6415*cos($Om - $Lp) + 26*sin($Om - $Lp);
$s += 5222*cos(2*($F + $D) - $L + $Om) + 15*sin(2*($F + $D) - $L + $Om);
$s += (168 - $T)*cos(2*$Lp) + 10*sin(2*$Lp);
$s += 3268*cos($L + 2*($F + $D + $Om)) + 19*sin($L + 2*($F + $D + $Om));
$s += 104*cos(2*($F - $L)) + 2*sin(2*($F - $L));
$s -= 3250*cos($Lp + 2*($F + $Om)) + 5*sin($Lp + 2*($F + $Om));
$s += 3353*cos(2*($F + $D) + $Om) + 14*sin(2*($F + $D) + $Om);
$s += 3070*cos(2*($F + $Om) - $Lp) + 4*sin(2*($F + $Om) - $Lp);
$s += 3272*cos(2*$D + $Om) + 4*sin(2*$D + $Om);
$s -= 3045*cos($L + 2*($F - $D) + $Om) + sin($L + 2*($F - $D) + $Om);
$s -= 2768*cos(2*($L + $F - $D + $Om)) + 4*sin(2*($L + $F - $D + $Om));
$s += 3041*cos(2*($D - $L) + $Om) - 5*sin(2*($D - $L) + $Om);
$s += 2695*cos(2*($L + $F) + $Om) + 12*sin(2*($L + $F) + $Om);
$s += 2719*cos(2*($F - $D) - $Lp + $Om) - 3*sin(2*($F - $D) - $Lp + $Om);
$s += 2720*cos($Om - 2*$D) - 9*sin($Om - 2*$D);
$s -= 51*cos(2*$D - $L - $Lp) - 4*sin(2*$D - $L - $Lp);
$s -= 2206*cos(2*($L - $D) + $Om) - sin(2*($L - $D) + $Om);
$s -= 199*cos($L + 2*$D) - 2*sin($L + 2*$D);
$s -= 1900*cos($Lp + 2*($F - $D) + $Om) - sin($Lp + 2*($F - $D) + $Om);
$s -= 41*cos($L - $Lp) - 3*sin($L - $Lp);
$s += 1313*cos(2*($F - $L + $Om)) - sin(2*($F - $L + $Om));
$s += 1233*cos(3*$L + 2*($F + $Om)) + 7*sin(3*$L + 2*($F + $Om));
$s -= 81*cos(2*$D - $Lp) - 2*sin(2*$D - $Lp);
$s += 1232*cos($L - $Lp + 2*($F + $Om)) + 4*sin($L - $Lp + 2*($F + $Om));
$s -= 20*cos($D) + 2*sin($D);
$s += 1207*cos(2*($F + $D + $Om) - $L - $Lp) + 3*sin(2*($F + $D + $Om) - $L - $Lp);
$s += 40*cos(2*$F - $L) - 2*sin(2*$F - $L);
$s += 1129*cos(2*($F + $D + $Om) - $Lp) + 5*sin(2*($F + $D + $Om) - $Lp);
$s += 1266*cos($Om - 2*$L) - 4*sin($Om - 2*$L);
$s -= 1062*cos($L + $Lp + 2*($F + $Om)) + 3*sin($L + $Lp + 2*($F + $Om));
$s -= 1129*cos(2*$L + $Om) + 2*sin(2*$L + $Om);
$s -= 9*cos($Lp + $D - $L);
$s += 35*cos($L + $Lp) - 2*sin($L + $Lp);
$s -= 107*cos($L + 2*$F) - sin($L + 2*$F);
$s += 1073*cos(2*($F - $D) - $L + $Om) - 2*sin(2*($F - $D) - $L + $Om);
$s += 854*cos($L + 2*$Om);
$s -= 553*cos($D - $L) + 139*sin($D - $L);
$s -= 710*cos(2*($F + $Om) + $D) + 2*sin(2*($F + $Om) + $D);
$s += 647*cos(2*($F + 2*$D + $Om) - $L) + 4*sin(2*($F + 2*$D + $Om) - $L);
$s -= 700*cos($Lp - $L + $D + $Om);
$s += 672*cos(2*($F - $Lp - $D) + $Om);
$s += 663*cos($L + 2*($F + $D) + $Om) + 4*sin($L + 2*($F + $D) + $Om);
$s -= 594*cos(2*($F - $L + $D + $Om)) + 2*sin(2*($F - $L + $D + $Om));
$s -= 610*cos(2*$Om - $L) - 2*sin(2*$Om - $L);
$s -= 556*cos($L + $Lp + 2*($F - $D + $Om));

$dEpsDeg = $s /36000000000.0;

return $dEpsDeg;

} // End of  dEps_2000B()


// ****************************************************************************

/*
   This function computes the calendrical JD number, which is always a positive
   integer (or zero).  This is the JD number used for simple, basic calendrical
   computations where the time of day can be ignored, such as when computing
   the exact number of whole calendar days between two dates.  For example,
   when we want to compute how many days it is until Christmas, we do not
   need to consider the time of day on either of the dates to perform the
   computation - we simply count whole days.

   This JD value  corresponds to the astronomical JD number for 12h, the
   midpoint of the given date.

   To obtain the astronomical JD number for 00h, the beginning of the date,
   simply subtract 0.5 from this integer value.  This is the value to which
   may later be added the fraction of a day corresponding to a given time so
   that we may obtain the general JD number for any given moment of the day.

   ISOymd  = ISO signed integer encoded date

   CalMode = Calendar Mode (0=Julian, 1=Gregorian=Default)

*/

   function JD_For_ISOymd ($ISOymd, $CalMode=1)
{

// ----------------------------------------------
// EXTRACT INDIVIDUAL DATE ARGUMENTS (y,m,d) FROM
// WITHIN ISO INTEGER ENCODED DATE ARGUMENT.

   $u = abs($ISOymd);
   $w = floor($u / 10000);
   $y = (($ISOymd < 0) ? -1 : 1) * $w;
   $m = floor(($u - 10000*$w) / 100);
   $d = ($u - 10000*$w - 100*$m);

// -------------------------------------------------
// COMPUTE THE JD NUMBER FOR 12h ON GIVEN DATE. THIS
// NUMBER WILL ALWAYS EQUATE TO AN INTEGER VALUE.

   $JD12 = ($CalMode == 0) ? JulianToJD($m,$d,$y) : GregorianToJD($m,$d,$y);

   return $JD12;

}

// ****************************************************************************

/*

   This is the inverse JD function.  Given the general JD number and calendar
   mode, it returns the corresponding ISO integer encoded calendar date.

   The JD argument is always positive, but is NOT restricted to only integer
   values and may contain a fractional component corresponding to the time
   of day.


   CalMode = Calendar Mode (0=Julian, 1=Gregorian=Default)

*/

   function ISOymd_For_JD ($JD, $CalMode=1)
{

// -----------------------------------------------
// COMPUTE JD12 FROM GIVEN JD AND THEN COMPUTE THE
// CORRSPONDING DATE STRING FOR JD12 IN "m/d/y"
// FORMAT.

   $JD12 = floor($JD + 0.5);

   $mdyStr = ($CalMode == 0) ? JDtoJulian($JD12) : JDtoGregorian($JD12);

// -----------------------------------------
// COMPUTE THE DATE STRING IN ISO ymd FORMAT
// FROM THE "m/d/y" DATE STRING VALUES.

   list($m, $d, $y) = preg_split("[/]", $mdyStr);

   $ISOymd = $y . sprintf("%02d", $m) . sprintf("%02d", $d);

   return $ISOymd;
}

// ****************************************************************************

/*
   This boolean function indicates whether or not a given date is valid.

   It returns TRUE if the date is valid or FALSE if the date is invalid.


   ISOymd  = ISO signed integer encoded date
   CalMode = Calendar Mode (0=Julian, 1=Gregorian=Default)

*/

   function Is_Valid_Date ($ISOymd, $CalMode=1)
{

// -----------------------------
// COMPUTE JD FOR 12:00 ON DATE.

   $JD12 = JD_For_ISOymd($ISOymd, $CalMode);

// ---------------------------------------------------------
// NOW, REVERSE THE PROCESS AND COMPUTE A CHECK DATE VALUE
// FROM THE PREVIOUSLY COMPUTED JD12 VALUE.

   $ISOymdCheck = ISOymd_For_JD($JD12, $CalMode);

// ------------------------------------------------------------------
// IF THE CHECK DATE VALUE PERFECTLY MATCHES THE INPUT DATE ARGUMENT,
// THEN THE DATE IS OK (TRUE), ELSE THE DATE IS INVALID (FALSE).

   if ($ISOymdCheck == $ISOymd) {return TRUE;} else {return FALSE;}

   return (($ISOymdCheck == $ISOymd) ? TRUE : FALSE);

} // End of  Is_Valid_Date()


// ****************************************************************************


/*
   This function computes the fraction of a day equivalent to a given time.
   The time can be a negative or positive value as indicated by SignValue.

   It is intended for times in the range from 0 to 24h


   SignValue  = 1 for a positive time  or  -1 for a negative time

   hh, mm, ss = Time elements (positive values, NOT restricted to integers)

*/

   function Days_For_HMS ($SignValue, $hh=0, $mm=0, $ss=0)
{

// -----------------------------------
// COMPUTE TIME EQUIVALENT IN SECONDS.

   $TimeInSeconds = bcAdd(bcAdd(bcMul("3600", "$hh", 18), bcMul("60", "$mm", 18)), "$ss", 18);

   $SignSymbol = ($SignValue < 0) ? "-" : "";

   if ($TimeInSeconds == 0) {$SignSymbol = "";}

// -----------------------------------------------
// COMPUTE TIME EQUIVALENT AS A FRACTION OF A DAY.

   $TimeInDays = bcDiv("$TimeInSeconds", "86400", 18);

// --------------------------------------------------
// ROUND OFF TimeInDays AT 16 DECIMALS AND THEN STRIP
// OFF ANY REDUNDANT ZEROS AND/OR DECIMAL POINT.

   $TimeInDays = bcAdd("$TimeInDays", "0.00000000000000005", 16);

   $TimeInDays = RTrim(RTrim($TimeInDays, "0"), ".");

   return "$SignSymbol$TimeInDays";
}

// ****************************************************************************

/*
 THIS FUNCTION TAKES A NUMERIC MONTH ARGUMENT FROM 01 TO 12 AND RETURNS THE
 CORRESPONDING ENGLISH MONTH NAME STRING FROM "January" TO "December".

 ANY ARGUMENT NOT RECOGNIZED AS A NUMBER FROM 1 TO 12 WILL RETURN AN EMPTY
 STRING AS AN ERROR INDICATOR.

*/
   function Month_String ($m)
{

// -------------------------
// DEFINE MONTH NAMES TABLE.

   $months = "January  Febuary  March    April    May      June     July     August   SeptemberOctober  November December ";

// -------------------------------------------
// IF MONTH ARGUMENT IS IN THE CORRECT NUMERIC
// RANGE, THEN RETURN THE MONTH NAME STRING.

// UNPAD ANY ZERO-PADDED MONTH VALUE.
   $w = intval(LTrim(trim($m), '0'));

   if ($w > 0 && $w < 13)
      {return trim(substr($months, 9*($w-1), 9));}
   else
      {return "";}


} // End of  Month_String()


// ****************************************************************************


/*
   This function returns the full, English weekday name string for the given
   ISO integer encoded date and calendar mode.


   ISOymd  = ISO signed integer encoded date
   CalMode = Calendar Mode (0=Julian, 1=Gregorian=Default)

*/

   function DoW_Str_For_ISOymd ($ISOymd, $CalMode)
{

// -----------------------------
// COMPUTE JD FOR 12:00 ON DATE.

   $JD12 = JD_For_ISOymd($ISOymd, $CalMode);

// ------------------------
// GET DoW STRING FOR JD12.

   return JDDayOfWeek ($JD12, 1);
}


// ****************************************************************************

   function Deg_For_DMS ($sign, $DMSStr)

{
   $dd = $mm = $ss = 0;

   $SignVal = ($sign === "-" || intval($sign) < 0) ? -1 : 1;

   $w = trim($DMSStr);  if ($w == "") {return "0";}

   $ang = preg_replace('/\s+/', ' ', $w);
   $ang = str_replace("+", "", $ang);
   $ang = str_replace("-", "", $ang);

   $w1 = preg_split("[ ]", $ang);  $AngCount = count($w1);

   @list($dd,$mm,$ss) = preg_split("[ ]", $ang);  $AngCount = count($w1);

   if ($AngCount >= 1) {$w2  = $dd * 3600;}
   if ($AngCount >= 2) {$w2 += $mm * 60;}
   if ($AngCount >= 3) {$w2 += $ss;}

   $AngDeg = $SignVal * ($w2 / 3600);

   return $AngDeg;

} // End of  Deg_For_DMS()



// ****************************************************************************

/*

   This function returns the degrees, minutees and seconds corresponding
   to a given positive or negative decimal degrees argument.

   PosSignFlag = This flag indicates whether or not to attach a positive sign
                 to positive values.

                 TRUE  = Attach a positive sign to all positive values
                 FALSE = Do NOT attach positive sign any positive values


   DegArg = The  decimal degrees argument (0 to 360 degrees).


   dddDigits = Number of digits in output degrees value (1 to 3)


   ssDecimals = Decimals at which to round off output seconds value (0 to 3)


   SymbolsFlag = This flag indicates whether or not to attach the symbols for
                 degrees, minutes and seconds ( ' ") to the output DMS string.

                 TRUE  = Attach DMS symbols to output DMS string
                 FALSE = Do NOT attach DMS symbols to output DMS string

*/

   function DMS_For_Deg ($PosSignFlag, $DegArg, $dddDigits, $ssDecimals, $SymbolsFlag=TRUE)
{

// -------------------------------
// HANDLE NUMERICAL SIGN OF INPUT.

   if ($DegArg < 0) {$sign = "-";} else {$sign = "";}

   if ($PosSignFlag === TRUE && $sign != "-")   {$sign = "+";}

// ----------------------------
// WORK WITH THE ABSOLUTE VALUE
// OF ORIGINAL ARGUMENT.  THE
// NUMERICAL SIGN WILL BE
// REAPPLIED TO THE OUTPUT.

   $DegArg = abs($DegArg);

// ----------------
// COMPUTE DEGREES.

   $ddd = floor($DegArg);

// --------------------
// COMPUTE ARC MINUTES.

   $w  = 60*($DegArg - $ddd);
   $mm = (int)$w;

// --------------------
// COMPUTE ARC SECONDS.

   $ss = 60*($w - $mm);

// ----------------------------------
// ADJUST REDUNDANT DDD PADDING ZEROS
// ACCORDING TO ($dddDigits) VALUE.

   if ($dddDigits == 1)
      {$ddd = 0+$ddd;}

   if ($dddDigits == 2)
      {
       if (StrLen($ddd) < 2) {$ddd = "0$ddd";}
      }

   if ($dddDigits == 3)
      {
       if (StrLen($ddd) < 2)  {$ddd = "00$ddd";}
       if (StrLen($ddd) == 2) {$ddd = "0$ddd";}
      }

// ------------------------------------------
// FORMAT ARC MINUTES VALUE ($mm) FOR OUTPUT.

   if ($mm < 10) {$mm = "0$mm";}

// ------------------------------------------
// CONSTRUCT OUTPUT DMS LABELS, IF INDICATED.

   $DDDsymbol = $MMsymbol = $SSsymbol = "";

   if ($SymbolsFlag === TRUE)
      {
       $DDDsymbol = "&deg;";
       $MMsymbol  = "'";
       $SSsymbol  = '"';
      }

// -------------------------------------------
// FORMAT ARC SECONDS VALUE ($ss). FOR OUTPUT.

   $SSformat = "%02." . "$ssDecimals" . "f";

   if ($ssDecimals == 0)
      {$ss = sPrintF("%02d", $ss);
      }
   else
      {
       $ss = sPrintF($SSformat, $ss);
       if ($ss < 10) {$ss = "0$ss";}
      }

// -------------------------------------------
// PATCH FOR THAT BLASTED RESIDUAL 60 PROBLEM.

   if ($ss == 60) {$mm = sprintf("%02d", $mm+1);   $ss = sPrintF($SSformat, 0);}
   if ($mm == 60) {$hh = sprintf("%02d", $hh+1);   $mm = "00";}

// --------------------------------------------------
// RETURN THE FORMATTED "DDD MM SS.sss" ANGLE STRING.

   return "$sign$ddd$DDDsymbol $mm$MMsymbol $ss$SSsymbol";

} // End of  DMS_For_Deg()


// ****************************************************************************

/*

   This function returns the hours, minutes and seconds corresponding to an
   angle argument given in decimal degrees.  360 degrees it taken as equivalent
   to exactly 24 hours which equates to 15 degrees per hour.


   PosSignFlag = This flag indicates whether or not to attach a positive sign
                 to positive output values.

                 TRUE  = Attach a positive sign to all positive output values
                 FALSE = Do NOT attach positive sign any positive output values


   DegArg = The  decimal degrees argument (0 to 360 degrees).


   hhDigits = Number of digits in output hours value (1 to 2)


   ssDecimals = Decimals at which to round off output seconds value (0 to 3)


   SymbolsFlag = This flag indicates whether or not to attach the symbols for
                 hours, minutes and seconds (h m s) to the output HMS string.

                 TRUE  = Attach HMS symbols to output HMS string
                 FALSE = Do NOT attach HMS symbols to output HMS string

*/

   function HMS_For_Deg ($PosSignFlag, $DegArg, $hhDigits, $ssDecimals, $SymbolsFlag=TRUE)
{

// -------------------------------
// HANDLE NUMERICAL SIGN OF INPUT.

   if ($DegArg < 0) {$sign = "-";} else {$sign = "";}

   if ($PosSignFlag === TRUE && $sign != "-")   {$sign = "+";}

// ----------------------------
// CONVERT DECIMAL DEGREES INTO
// DECIMAL HOURS AND WORK WITH
// THE ABSOLUTE VALUE.  THE
// SIGN WILL BE REAPPLIED TO
// THE OUTPUT.

   $deg = abs($DegArg / 15);

// --------------
// COMPUTE HOURS.

   $hh = floor($deg);

// ----------------
// COMPUTE MINUTES.

   $w  = 60*($deg - $hh);

   $mm = floor($w);

// ----------------
// COMPUTE SECONDS.

   $ss = 60*($w - $mm);

// --------------------------------
// PATCH TO HANDLE THE 60s PROBLEM.

   if ($ss == 60) {$mm += 1;  $ss = 0;}
   if ($mm == 60) {$hh += 1;  $mm = 0;}

// ------------------------------
// FORMAT HOURS VALUE FOR OUTPUT.

   $hh = sPrintF("%02d", $hh);

// --------------------------------
// FORMAT MINUTES VALUE FOR OUTPUT.

   $mm = sPrintF("%02d", $mm);

// --------------------------------------------
// ATTACH OUTPUT SYMBOLS (H M S), IF INDICATED.

   $HHsymbol = $MMsymbol = $SSsymbol = "";

   if ($SymbolsFlag)
      {
       $HHsymbol = "h";
       $MMsymbol = "m";
       $SSsymbol = "s";
      }

// --------------------------------
// FORMAT SECONDS VALUE FOR OUTPUT.

   $SSformat = "%02." . "$ssDecimals" . "f";
   $ss = sPrintF($SSformat, $ss);
         if ($ss < 10) {$ss = "0$ss";}

// ------------------------------------------------
// RETURN THE FORMATTED "HH MM SS.sss" TIME STRING.

   return "$sign$hh" . "$HHsymbol $mm" .  "$MMsymbol $ss" . "$SSsymbol";

} // End of  HMS_For_Deg()


// ****************************************************************************


/*

   Return hours, minutes and seconds corresponding to a given decimal days
   value.  The argument can be either negative or positive.

   The absolute value of the returned result can be > 24 hours.

   The returned seconds value may be rounded off to up to 3 decimals. If no
   decimals are specified, then the default zero value will be used.

*/

   function HMS_For_Days ($DaysVal, $NumDecimals=0)

{

// ---------------------------
// READ INPUT ARGUMENT VALUES.

   $days = trim($DaysVal);

   $d = floor(abs((trim($NumDecimals))));

// --------------------------------------
// IF DAYS ARGUMENT IS EMPTY, THEN SET TO
// DEFAULT ZERO VALUE.

   if ($days == "") {$days = 0;}

// ---------------------------------------
// IF DECIMALS ARGUMENT IS EMPTY, THEN SET
// TO DEFAULT ZERO VALUE.

   if ($d == "") {$d = 0;}

// ---------------------
// IF DECIMALS > 3, THEN
// SET TO DEFAULT 3 LIMIT.

   if ($d > 3) {$d = 3;}

// ---------------------------
// REMEMBER ANY NEGATIVE SIGN.

   if ($days < 0) {$sign = "-";} else {$sign = "";}

// --------------------------------------------
// WORK WITH ABSOLUTE days VALUE.  ANY NEGATIVE
// SIGN WILL BE PRESERVED IN THE RETURNED RESULT.

   $days = abs($days);

// --------------------
// COMPUTE HOURS VALUE.

   $w   = $days * 24;

   $hh  = floor($w);

// ----------------------
// COMPUTE MINUTES VALUE.

   $w   = 60*($w - $hh);

   $mm  = floor($w);

// ----------------------
// COMPUTE SECONDS VALUE.

   $ss  = 60*($w - $mm);

// -------------------------------------
// ROUND SECONDS VALUE (ss) TO THE GIVEN
// NUMBER OF DECIMALS (d).

   $ss = sprintf ("%01.$d" . "f", $ss);

   if ($ss < 10) {$ss = "0$ss";}

// --------------------------------------
// ACCOUNT FOR THAT ANNOYING 60S PROBLEM.

   if ($ss == 60) {$mm += 1;  $ss = 0;}
   if ($mm == 60) {$hh += 1;  $mm = 0;}

// ------------------------------------
// FORMAT THE TIME ELEMENTS FOR OUTPUT.

   $hh = sprintf("%02d", $hh);
   $mm = sprintf("%02d", $mm);

   $ss = sprintf ("%01.$d" . "f", $ss);

   if ($ss < 10) {$ss = "0$ss";}

// -----------------------------------
// ATTACH HMS SYMBOLS TO TIME ELEMENTS
// AND RECALL NUMERICAL SIGN.

   $hh .= "h";  $mm .= "m";  $ss .= "s";

   return "$sign$hh $mm $ss";

} // End of  HMS_For_Days()


// ****************************************************************************

/*

   This function returns the correction in degrees to be  applied to the
   ecliptical longitude to reduce it from the VSOP87 dynamical system to
   the FK5 system of coordinates.

*/

   Function FK5_Lng_Corr ($JD, $EclLngDeg, $EclLatDeg)

{

// -----------------------------------------
// COMPUTE TIME RECKONED IN JULIAN CENTURIES
// FROM J2000.0 CORRESPONDING TO THE JD

   $T = ($JD - 2451545.0) / 36525;

// ----------------------------------
// READ ECLIPTICAL LATITUDE ARGUMENT.

   $b = deg2rad($EclLatDeg);

   $Lprime = deg2rad($EclLngDeg - 1.397*$T - 0.00031*$T*$T);

// ----------------------------------------
// COMPUTE LONGITUDE CORRECTION IN DEGREES.

   $w = (0.03916*(cos($Lprime) + sin($Lprime))*tan($b) - 0.09033) / 3600;

// --------------------
// RETURN THE LONGITUDE
// CORRECTION.

   return $w;

} // End of  FK5_Lng_Corr()


/*
   --------------------------------------------
   THIS FUNCTION RETURNS THE CORRECTION TO BE
   APPLIED TO THE ECLIPTICAL LATITUDE TO REDUCE
   IT FROM THE VSOP87 DYNAMICAL SYSTEM TO THE
   FK5 SYSTEM OF COORDINATES.

*/

   Function FK5_Lat_Corr ($JD, $EclLngDeg)
{

// -----------------------------------------
// COMPUTE TIME RECKONED IN JULIAN CENTURIES
// FROM J2000.0 CORRESPONDING TO THE JD

   $T = ($JD - 2451545) / 36525;

// ------------------------------------------
// COMPUTE AUXILIARY WORKING ANGLEIN RADIANS.

   $w = deg2rad($EclLngDeg - 1.397*$T - 0.00031*$T*$T);

// -------------------------------------------------
// DONE.  RETURN THE LATITUDE CORRECTION IN DEGREES.

   return 0.03916*(cos($w) - sin($w)) / 3600;

} // End of  FK5_Lat_Corr()


// ****************************************************************************


   function Ecl_To_Equ_Coords ($EclLngDeg, $EclLatDeg, $EpsDeg)

  {

// ------------------------------
// CONVERT INPUT ECLIPTICAL ANGLE
// ARGUMENTS INTO RADIANS.

   $LngRad = deg2rad($EclLngDeg);
   $LatRad = deg2rad($EclLatDeg);
   $EpsRad = deg2rad($EpsDeg);

// -------------------------------------
// COMPUTE THE EQUATORIAL RA IN DEGREES.

   $y = sin($LngRad)*cos($EpsRad) - tan($LatRad)*sin($EpsRad);

   $x = cos($LngRad);

   $RADeg = rad2deg(atan2($y, $x));

   if ($RADeg < 0) {$RADeg += 360;}

// -----------------------------------
// COMPUTE THE EQUATORIAL DECLINATION.

   $DeclRad = asin(sin($LatRad)*cos($EpsRad) + cos($LatRad)*sin($LngRad)*sin($EpsRad));

   $DeclDeg = rad2deg($DeclRad);

// ---------------------------------------------
// DONE.  RETURN THE RA AND DECLINATION, BOTH IN
// DEGREES, IN A SPACE DELIMITED DATA VECTOR.

   return "$RADeg $DeclDeg";

  } // End of  Ecl_To_Equ_Coords()


// ****************************************************************************


/*

   Compute the mean  obliquity of the ecliptic in degrees for the
   current JD argument corresponding to any given moment.

   This ecliptic obliquity function is based on a formula derived
   by J. Laskar, Astronomy and Astrophysics, 1968, Vol. 157, p68.

   The estimated accuracy of this formula is 0.01 arc second between
   the years 1000 AD and 3000 AD and a few arc seconds after 10,000
   years.  It is only valid in the range 10,000 years either way
   of J2000.0

   Over the long term, it is more accurate than the formula
   adopted by the International Astronomical Union (IAU)
   for general ephemeris computations.

   ---------
   ARGUMENTS

   $JD = General JD argument for any date and time.


   ------
   ERRORS

   No error checking is done.

*/


   function Epsilon_Mean ($JD)

{

// ----------------------------------------------
// COMPUTE THE TIME IN JULIAN CENTURIES RECKONED
// FROM J2000.0 CORRESPONDING TO THE JD ARGUMENT
// AND THEN DIVIDE BY 100 FOR DECIMILLENNIA.

   $t = ($JD - 2451545.0) / 36525 / 100;

// --------------------------------------
// COMPUTE MEAN OBLIQUITY OF THE ECLIPTIC
// EXPRESSED IN ARC SECONDS.

   $p = $t*$t;

   $eMeanSS = 84381.448 - 4680.93*$t;

   $eMeanSS -=    1.55*$p;  $p *= $t;
   $eMeanSS += 1999.25*$p;  $p *= $t;
   $eMeanSS -=   51.38*$p;  $p *= $t;
   $eMeanSS -=  249.67*$p;  $p *= $t;
   $eMeanSS -=   39.05*$p;  $p *= $t;
   $eMeanSS +=    7.12*$p;  $p *= $t;
   $eMeanSS +=   27.87*$p;  $p *= $t;
   $eMeanSS +=    5.79*$p;  $p *= $t;
   $eMeanSS +=    2.45*$p;

// -----------------------------------------------
// RETURN THE MEAN OBLIQUITY EXPRESSED IN DEGREES.

   $eMeanDeg = $eMeanSS / 3600;

   return $eMeanDeg;

} // End of  Epsilon_Mean()



// ****************************************************************************


/*

   Compute the nutation in Epsilon, Delta Epsilon or
   nutation in obliquity of the ecliptic, in degrees.

   This is the correction to apply to the MEAN value
   of the obliquity to obtain the TRUE obliquity.

   The general precision level is  0.001 arc sec.

   This function is based on the 1980 IAU theory.

   $JD = JD number corresponding to any date and time

   $t  = Julian time factor as reckoned from J2000
   $t2 = $t to the power of 2
   $t3 = $t to the power of 3

   ==============================================
   Nutation correction elements (1980 IAU Theory)

   $w1 = Mean elongation of the moon from the sun
   $w2 = Mean anomaly of the sun
   $w3 = Mean anomaly of the moon
   $w4 = Argument of latitude of the moon
   $w5 = Longitude of lunar ascending node

   $w  = Nutation series accumulator

*/

   Function Delta_Epsilon ($JD)
{

// COMPUTE TIME FACTOR IN JULIAN CENTURIES AS
// RECKONED FROM J2000 AND ITS POWERS.

   $T  = ($JD - 2451545) / 36525;
   $T2 = $T * $T;
   $T3 = $T * $T * $T;

// -----------------------------------------
// Compute mean lunar elongation in radians.

   $w1 = 297.85036 + 445267.11148*$T - 0.0019142*$T2 + ($T3 / 189474);
   $w1 = deg2rad($w1);

// --------------------------------------
// COMPUTE MEAN SOLAR ANOMALY IN RADIANS.

   $w2 = 357.52772 + 35999.05034*$T - 0.0001603*$T2 - ($T3 / 300000);
   $w2 = deg2rad($w2);

// --------------------------------------
// COMPUTE MEAN LUNAR ANOMALY IN RADIANS.

   $w3 = 134.96298 + 477198.867398*$T + 0.0086972*$T2 + ($T3 / 56250);
   $w3 = deg2rad($w3);

// ----------------------------------------------
// COMPUTE LUNAR ARGUMENT OF LATITUDE IN RADIANS.

   $w4 = 93.27191 + 483202.017538*$T - 0.0036825*$T2 + ($T3 / 327270);
   $w4 = deg2rad($w4);

// -----------------------------------------------------
// COMPUTE LONGITUDE OF LUNAR ASCENDING NODE IN RADIANS.

   $w5 = 125.04452 - 1934.136261*$T + 0.0020708*$T2 + ($T3 / 450000);
   $w5 = deg2rad($w5);

// ---------------------------------------------------
// COMPUTE THE NUTATION IN OBLIQUITY (ArcSec * 10000).

   $w  = cos($w5)*(92025 + 8.9*$T);
   $w += cos(2*($w4 - $w1 + $w5))*(5736 - 3.1*$T);
   $w += cos(2*($w4 + $w5))*(977 - 0.5*$T);
   $w += cos(2*$w5)*(0.5*$T - 895);
   $w += cos($w2)*(54 - 0.1*$T);
   $w += cos($w2 + 2*($w4 - $w1 + $w5))*(224 - 0.6*$T);
   $w += cos($w3 + 2*($w4 + $w5))*(129 - 0.1*$T);
   $w += cos(2*($w4 - $w1 + $w5) - $w2)*(0.3*$T - 95);
   $w += 200*cos(2*$w4 + $w5);
   $w -= 70*cos(2*($w4 - $w1) + $w5);
   $w -= 53*cos(2*($w4 + $w5) - $w3);
   $w -= 33*cos($w3 + $w5);
   $w += 26*cos(2*($w1 + $w4 + $w5) - $w3);
   $w += 32*cos($w5 - $w3);
   $w += 27*cos($w3 + 2*$w4 + $w5);
   $w -= 24*cos(2*($w4 - $w3) + $w5);
   $w += 16*cos(2*($w1 + $w4 + $w5));
   $w += 13*cos(2*($w3 + $w4 + $w5));
   $w -= 12*cos($w3 + 2*($w4 - $w1 + $w5));
   $w -= 10*cos(2*$w4 + $w5 - $w3);
   $w -= 8*cos(2*$w1 - $w3 + $w5);
   $w += 7*cos(2*($w2 - $w1 + $w4 + $w5));
   $w -= 7*cos($w3);
   $w += 9*cos($w2 + $w5);
   $w += 7*cos($w3 + $w5 - 2*$w1);
   $w += 6*cos($w5 - $w2);
   $w += 5*cos(2*($w1 + $w4) - $w3 + $w5);
   $w += 3*cos($w3 + 2*($w4 + $w1 + $w5));
   $w -= 3*cos($w2 + 2*($w4 + $w5));
   $w += 3*cos(2*($w4 + $w5) - $w2);
   $w += 3*cos(2*($w1 + $w4) + $w5);
   $w -= 3*cos(2*($w3 + $w4 + $w5 - $w1));
   $w -= 3*cos($w3 + 2*($w4 - $w1) + $w5);
   $w += 3*cos(2*($w1 - $w3) + $w5);
   $w += 3*cos(2*$w1 + $w5);
   $w += 3*cos(2*($w4 - $w1) + $w5 - $w2);
   $w += 3*cos($w5 - 2*$w1);
   $w += 3*cos(2*($w3 + $w4) + $w5);

// ----------------------------
// RETURN NUTATION IN OBLIQUITY
// EXPRESSED IN DEGREES.

   return $w / 36000000;

} // End of  Delta_Epsilon()


// ****************************************************************************

/*

   This unction computes the nutation in ecliptical longitude,
   in degrees, for any given JD argument.

*/

   function Delta_Psi ($JD)
{

// -------------------------------------------------------
// COMPUTE POWERS OF TIME IN JULIAN CENTURIES FROM J2000.0

   $T1 = ($JD - 2451545) / 36525;
   $T2 = $T1*$T1;
   $T3 = $T2*$T1;

// -----------------------------------------
// COMPUTE MEAN LUNAR ELONGATION IN RADIANS.

   $w1 = 297.85036 + 445267.11148*$T1 - 0.0019142*$T2 + ($T3 / 189474);
   $w1 = deg2rad($w1);

// --------------------------------------
// COMPUTE MEAN SOLAR ANOMALY IN RADIANS.

   $w2 = 357.52772 + 35999.05034*$T1 - 0.0001603*$T2 - ($T3 / 300000);
   $w2 = deg2rad($w2);

// --------------------------------------
// COMPUTE MEAN LUNAR ANOMALY IN RADIANS.

   $w3 = 134.96298 + 477198.867398*$T1 + 0.0086972*$T2 + ($T3 / 56250);
   $w3 = deg2rad($w3);

// ----------------------------------------------
// COMPUTE LUNAR ARGUMENT OF LATITUDE IN RADIANS.

   $w4 = 93.27191 + 483202.017538*$T1 - 0.0036825*$T2 + ($T3 / 327270);
   $w4 = deg2rad($w4);

// ----------------------------------------
// COMPUTE LUNAR ASCENDING NODE IN RADIANS.

   $w5 = 125.04452 - 1934.136261*$T1 + 0.0020708*$T2 + ($T3 / 450000);
   $w5 = deg2rad($w5);

// ---------------------------------------------
// COMPUTE THE NUTATION IN ECLIPTICAL LONGITUDE.

   $w  = sin($w5)*(-174.2*$T1 - 171996);
   $w += sin(2*($w4 + $w5 - $w1))*(-1.6*$T1 - 13187);
   $w += sin(2*($w4 + $w5))*(-2274 - 0.2*$T1);
   $w += sin(2 * $w5)*(0.2*$T1 + 2062);
   $w += sin($w2)*(1426 - 3.4*$T1);
   $w += sin($w3)*(0.1*$T1 + 712);
   $w += sin(2*($w4 + $w5 - $w1) + $w2)*(1.2*$T1 - 517);
   $w += sin(2 * $w4 + $w5)*(-0.4*$T1 - 386);
   $w += sin(2*($w4 + $w5 - $w1) - $w2)*(217 - 0.5*$T1);
   $w += sin(2*($w4 - $w1) + $w5)*(129 + 0.1*$T1);
   $w += sin($w3 + $w5)*(0.1*$T1 + 63);
   $w += sin($w5 - $w3)*(-0.1*$T1 - 58);
   $w += sin(2*$w2)*(17 - 0.1*$T1);
   $w += sin(2*($w2 + $w4 + $w5 - $w1))*(0.1*$T1 - 16);
   $w -= 301*sin(2*($w4 + $w5) + $w3);
   $w -= 158*sin($w3 - 2*$w1);
   $w += 123*sin(2*($w4 + $w5) - $w3);
   $w +=  63*sin(2*$w1);
   $w -=  59*sin(2*($w1 + $w4 + $w5) - $w3);
   $w -=  51*sin(2 * $w4 + $w3 + $w5);
   $w +=  48*sin(2*($w3 - $w1));
   $w +=  46*sin(2*($w4 - $w3) + $w5);
   $w -=  38*sin(2*($w1 + $w4 + $w5));
   $w -=  31*sin(2*($w3 + $w4 + $w5));
   $w +=  29*sin(2*$w3);
   $w +=  29*sin(2*($w4 + $w5 - $w1) + $w3);
   $w +=  26*sin(2*$w4);
   $w -=  22*sin(2*($w4 - $w1));
   $w +=  21*sin(2*$w4 + $w5 - $w3);
   $w +=  16*sin(2*$w1 - $w3 + $w5);
   $w -=  15*sin($w2 + $w5);
   $w -=  13*sin($w3 + $w5 - 2*$w1);
   $w -=  12*sin($w5 - $w2);
   $w +=  11*sin(2*($w3 - $w4));
   $w -=  10*sin(2*($w4 + $w1) + $w5 - $w3);
   $w -=   8*sin(2*($w4 + $w1 + $w5) + $w3);
   $w +=   7*sin(2*($w4 + $w5) + $w2);
   $w -=   7*sin($w3 - 2*$w1 + $w2);
   $w -=   7*sin(2*($w4 + $w5) - $w2);
   $w -=   7*sin(2*$w1 + 2*$w4 + $w5);
   $w +=   6*sin(2*$w1 + $w3);
   $w +=   6*sin(2*($w3 + $w4 + $w5 - $w1));
   $w +=   6*sin(2*($w4 - $w1) + $w3 + $w5);
   $w -=   6*sin(2*($w1 - $w3) + $w5);
   $w -=   6*sin(2*$w1 + $w5);
   $w +=   5*sin($w3 - $w2);
   $w -=   5*sin(2*($w4 - $w1) + $w5 - $w2);
   $w -=   5*sin($w5 - 2*$w1);
   $w -=   5*sin(2*($w3 + $w4) + $w5);
   $w +=   4*sin(2*($w3 - $w1) + $w5);
   $w +=   4*sin(2*($w4 - $w1) + $w2 + $w5);
   $w +=   4*sin($w3 - 2*$w4);
   $w -=   4*sin($w3 - $w1);
   $w -=   4*sin($w2 - 2*$w1);
   $w -=   4*sin($w1);
   $w +=   3*sin(2 * $w4 + $w3);
   $w -=   3*sin(2*($w4 + $w5 - $w3));
   $w -=   3*sin($w3 - $w1 - $w2);
   $w -=   3*sin($w2 + $w3);
   $w -=   3*sin(2*($w4 + $w5) + $w3 - $w2);
   $w -=   3*sin(2*($w1 + $w4 + $w5) - $w2 - $w3);
   $w -=   3*sin(2*($w4 + $w5) + 3*$w3);
   $w -=   3*sin(2*($w1 + $w4 + $w5) - $w2);

// ---------------------------
// RETURN ECLIPTICAL LONGITUDE
// EXPRESSED IN DEGREES.

   $dPsi = $w / 36000000;

   return $dPsi;

  } // End of  Delta_Psi()



// ****************************************************************************

/*
   This function computes the nutation in right ascension at the celestial
   equator, in degrees, also known as the equation of the equinoxes.

   THIS IS ALSO CALLED THE EQUATION OF THE EQUINOXES AND
   IS THE DIFFERENCE (True Equinox - Mean Equinox).

   NOTE:
   Degrees * 240 = Seconds of RA

*/

   function Delta_Equinox ($JD)
{

// ---------------------------------------------
// COMPUTE COSINE OF TRUE OBLIQUITY OF ECLIPTIC.

   $CosEpsTrue = cos(deg2rad(Epsilon_Mean($JD) + Delta_Epsilon($JD)));

// ---------------------------------------------------
// DONE.  RETURN EQUATION OF THE EQUINOXES IN DEGREES.

   return Delta_Psi($JD) * $CosEpsTrue;

} // End of  Delta_Equinox()


// ****************************************************************************

/*
   Given a distance in AUs, this function computes the equivalent distances
   in astronomical units (AUs), kilometers(km) and miles (mi).

   Mkm, Mmi, Bkm and Bmi mean millions and billions of kilometers or miles
   respectively.

   For clarity:
   1 million = 1E6
   1 billion = 1E9



*/

   function AU_To_AUKmMi ($AUs, $AUDecimals=7)

{

   $AU = trim($AUs);

   $km = $AU * 149597870.691;
   $mi = $km / 1.609344;

// ---------------------------------------------------------
// DETERMINE PREFIX: M=Millions or B=Billions of KILOMETERS.

   $factor = 1;
   $prefix = "";

   if ($km >= 1E6)  {$prefix = "M";  $factor = 1000000;}

   if ($km >= 1E9)  {$prefix = "B";  $factor = 1000000000;}

   $km /= $factor;

   $km = sprintf("%3.6f", $km) . " $prefix" . "km";

   $km = str_replace("| 0", "|  ", $km);

// ----------------------------------------------------
// DETERMINE PREFIX: M=Millions or B=Billions of MILES.

   $factor = 1;
   $prefix = "";

   if ($mi >= 1E6)  {$prefix = "M";  $factor = 1000000;}

   if ($mi >= 1E9)  {$prefix = "B";  $factor = 1000000000;}

   $mi /= $factor;

   $mi = sprintf("%3.6f", $mi) . " $prefix" . "mi";

   if ($AU >= 4) {$w = "7";} else {$w = "$AUDecimals";}

       $AU = sPrintF("%1.$w" . "f", $AU) . " AU";

   $LTd = $AU * 0.0057755183310895;

   $LTHMS = HMS_For_Days ($LTd, 3);

   $LTHMS = str_replace("00h ", "", $LTHMS);

   $w = "$AU =|$km =|$mi";

// ------------
// ZERO FILTERS

   $w = str_replace("AU|00",  "AU|  ",  $w);
   $w = str_replace("AU|0",   "AU| ",   $w);
   $w = str_replace("Mkm|00", "Mkm|  ", $w);
   $w = str_replace("Mkm|0",  "Mkm| ",  $w);
   $w = str_replace("Bkm|00", "Bkm|  ", $w);
   $w = str_replace("Bkm|0",  "Bkm| ",  $w);
   $w = str_replace("|", " ", $w);

   $w = str_replace("    ", "   ", $w);

// --------------
// Final filters.

   $w = str_replace("= 0",  "=  ",  $w);
   $w = str_replace("=  0", "=   ", $w);

// -----
// Done.

   return $w;

} // End of  AU_To_AUKmMi()


// ****************************************************************************

// This function returns the light-time in HMS
// corresponding to a given argument in AUs.

   function Light_Time_For_AU ($AU, $HMSFlag=TRUE)

{

// ---------------------------
// COMPUTE LIGHT-TIME IN DAYS.

   $LTDays = $AU * 0.0057755183310895;

// --------------------------------------------
// RETURN VALUE IN DAYS IF HMSFlag IS NOT TRUE.

   if ($HMSFlag !== TRUE) {return $LTDays;}


// ---------------------------------
// CONVERT LIGHT-TIME TO HMS STRING.

   $LTHMS = HMS_For_Days ($LTDays, 3);

// ------------------------------------------
// CUT OFF HOURS VALUE IF IT EQUATES TO ZERO.

   $LTHMS = str_replace("00h ", "", $LTHMS);

   return $LTHMS;
}

// ****************************************************************************

/*

This function converts any of several units of length or
distance into the equivalent in meters.  It makes use of
the metric length equivalents defined within this module.

It is the opposite of the Meters_To_Length() function.

----
NOTE

These computations are able to accomodate arbitrary
precision out to 105 decimals which is sufficient
for any practical usage.

---------
ARGUMENTS

$LengthVal = Numerical value of the length to be converted
$FromUnits = The units symbol corresponding to the value

The returned output is simply the equivalent length in meters.

-------------------------------------------------------
There are 16 units symbols recognized by this function.

They are:

 SYMBOL         UNITS

   A            Angstroms
   nm           Nanometers
 um or u        Micrometers (or Microns)
   mm           Millimeters
   cm           Centimeters
   in           Inches
   ft           Feet
   yd           Yards
   m            Meters
   km           Kilometers
   mi           Miles (Statute)
   nmi          Nautical miles
   LS           Light Seconds
   AU           Astronomical Units
   LY           Light Years (Based on Julian year)
   psc          Parsecs

IMPORTANT NOTE:
The units symbols used here ARE case sensitive.

ERRORS:
An error occurs if the argument cannot be resolved
into a valid numerical value and units symbol.  On
error, FALSE is returned.

*/

   Function Length_To_Meters ($LengthVal, $FromUnits)
{

// ------------------------------
// Set arbitrary precision limit.

   $nDec = 105;

// -----------------------------
// Get the numerical value part.
// Error if not a numerical value.

   $x = trim($LengthVal);

   if ($x == "") {$x = "0.0";}

   if (!is_numeric($x)) {return FALSE;}

   if (StrPos($x, ".") === FALSE) {$x = "$x.0";}

// ----------------
// Get units symbol.

   $UnitsSymbol = trim($FromUnits);

// ---------------------------------------------------
// Initialize factor to empty string.   If it is still
// empty on exit from the switch block, below then it
// means that the units symbol was not recognized as
// valid.

   $f = "";

// --------------------------------------
// Determine the conversion factor to use
// from the units symbol argument.
//
// NOTE: These symbols ARE case sensitive.

   switch ($UnitsSymbol)
  {
   case "A"   : $f = M_PER_A;    break;
   case "um"  : $f = M_PER_UM;   break;
   case "u"   : $f = M_PER_UM;   break;
   case "nm"  : $f = M_PER_NM;   break;
   case "mm"  : $f = M_PER_MM;   break;
   case "cm"  : $f = M_PER_CM;   break;
   case "in"  : $f = M_PER_IN;   break;
   case "ft"  : $f = M_PER_FT;   break;
   case "yd"  : $f = M_PER_YD;   break;
   case "m"   : $f = M_PER_M;    break;
   case "km"  : $f = M_PER_KM;   break;
   case "mi"  : $f = M_PER_MI;   break;
   case "nmi" : $f = M_PER_NMI;  break;
   case "LS"  : $f = CLIGHT;     break;
   case "AU"  : $f = M_PER_AU;   break;
   case "LY"  : $f = M_PER_LY;   break;
   case "pc"  : $f = M_PER_PC;   break;
  }

// ------------------------------------------
// Error if conversion factor is still empty,
// which means that the units symbol is not
// found in the table of valid symbols.
   If ($f === FALSE) {return $f;}

// ----------------------------------------------
// Otherwise, return the equivalent meters value
// by multiplying the value of the argument by
// the conversion factor $f.
   $w = bcMul("$x", "$f", $nDec);

// -----
// Done.
   return RTrim(RTrim($w, "0"), ".");

}  // End of Length_To_Meters()


// ****************************************************************************


/*

This function converts a length or distance value given in
meters into its equivalent in any of the 16 length units
recognized by this module.

It is the opposite of the Length_To_Meters() function and
recognizes the same length units symbols.

NOTE:
The computations are able to accomodate arbitrary
precision out to 105 decimals which is sufficient
for any practical usage.

ARGUMENTS
$metersArg =  Value in meters that will be converted
              to the specified output units.

$toUnits   =  Units symbol for the output value.  This
              symbol will NOT be attached to the output.
              It will simply be a numeric value that is
              understood to be in the specified units.

There are 16 units symbols recognized by this function.
They are:

 SYMBOL         UNITS

   A            Angstroms
   nm           Nanometers
 um or         Micrometers (or Microns)
   mm           Millimeters
   cm           Centimeters
   in           Inches
   ft           Feet
   yd           Yards
   m            Meters
   km           Kilometers
   mi           Miles (Statute)
   nmi          Nautical miles
   LS           Light Seconds
   AU           Astronomical Units
   LY           Light Years (Based on Julian year)
   pc           Parsecs
   LS           Light Seconds

IMPORTANT NOTE
The units symbols used here ARE case sensitive.

ERRORS
An error occurs if the argument cannot be resolved
into a valid numerical value and units symbol.  On
error, FALSE is returned.

*/


   Function Meters_To_Length ($MetersVal, $ToUnits)

{

// ------------------------------
// Set arbitrary precision limit.

   $nDec = 105;

// ------------------------------
// Read value of meters argument.

   $m = Trim("$MetersVal");

// -------------------------------------
// Error if meters value is non-numeric.

   If (!Is_Numeric($m)) {Return FALSE;}

   if (StrPos($m, ".") === FALSE) {$m = "$m.0";}

// --------------------------------------
// Read the output units symbol argument.

   $UnitsSymbol = Trim($ToUnits);

// ---------------------------------------------------
// Initialize factor to empty string.   If it is still
// empty on exit from the switch block below, then it
// means that the units symbol was not found in the
// table of valid units.

   $f = "";

// --------------------------------------
// Determine the conversion factor to use
// from the units symbol argument.
// THESE SYMBOLS ARE CASE SENSITIVE.

   switch ($UnitsSymbol)
  {
   case "A"   : $f = M_PER_A;    break;
   case "um"  : $f = M_PER_UM;   break;
   case "u"   : $f = M_PER_UM;   break;
   case "nm"  : $f = M_PER_NM;   break;
   case "mm"  : $f = M_PER_MM;   break;
   case "cm"  : $f = M_PER_CM;   break;
   case "in"  : $f = M_PER_IN;   break;
   case "ft"  : $f = M_PER_FT;   break;
   case "yd"  : $f = M_PER_YD;   break;
   case "m"   : $f = M_PER_M;    break;
   case "km"  : $f = M_PER_KM;   break;
   case "mi"  : $f = M_PER_MI;   break;
   case "nmi" : $f = M_PER_NMI;  break;
   case "LS"  : $f = CLIGHT;     break;
   case "AU"  : $f = M_PER_AU;   break;
   case "LY"  : $f = M_PER_LY;   break;
   case "pc"  : $f = M_PER_PC;   break;
  }

// ------------------------------------------
// Error if conversion factor is still empty.

   If ($f == "") {return FALSE;}

// ----------------------------------------------
// Otherwise, return the equivalent of the given
// meters value by dividing the value of the
// argument by the conversion factor $f.

   $w = bcDiv ($m, $f, $nDec);

   return RTrim(RTrim($w, "0"), ".");

}  // End of Meters_To_Length()


// ****************************************************************************

/*

   This function checks to see if a given length units symbol is among
   those currently recognized.  This list can be modified as needed.

   It is a Boolean test and returns TRUE if the units symbol is valid
   or FALSE otherwise.

*/
   function Is_Valid_Length_Unit ($UnitsSymbolStr)

{

   $UnitsSymbols = "A  NM AU U  MU MM CM IN FT YD M  KM MI NMILS AU LY PC ";

   $u = trim(strtoupper($UnitsSymbolStr));

   for ($i=0;   $i < 18;   $i++)

   {
    if ($u == trim(substr($UnitsSymbols, 3*$i, 3))) {return TRUE;};
   }

   return FALSE;

}

// ****************************************************************************

/*

 This function converts from one length unit into another, such as feet to
 centimeters, yards to meters, miles to kilometers, etc.

*/

   function From_Units_To_Units ($FromValStr, $FromUnitsStr, $ToUnitsStr)

 {
   $L1 = Length_To_Meters ($FromValStr, $FromUnitsStr);
   $w  = Meters_To_Length ($L1, $ToUnitsStr);

   $L2 = bcRound($w, 16);

   if (abs($w) < 1E-08) {$L2 = bcRound($w, 24);}
   if (abs($w) < 1E-16) {$L2 = bcRound($w, 32);}
   if (abs($w) < 1E-24) {$L2 = bcRound($w, 40);}

   $L2 = RTrim(RTrim($L2, "0"), ".");

   return $L2;

 } // End of  From_Units_To_Units()

// ****************************************************************************

/*
   This function converts from one temperature scale into another.  The scales
   recognized are the Fahrenheit (F), Celsius (C), Kelvin (K) and Rankine (R)
   scales.

   $TemperatureValue = Numerical value of the temperature
          $FromScale = From scale symbol (F,C,K,R)
          $ToScale   = To scale symbol (F,C,K,R)
*/


   function Temp_To_Temp ($TemperatureValue, $FromScale, $ToScale)

{

// -----------------------------------
// GET NUMERICAL VALUE OF TEMPERATURE.
// ERROR IF NON-NUMERIC.

   $t  = trim($TemperatureValue);

   if (is_numeric($t))

  {

// --------------------------------------
// CONVERT INPUT ($FromScale) TO K SCALE.

   switch (StrToUpper(trim($FromScale)))
  {
   case "F" : $t = (5*($t - 32) / 9) + 273.15;  break;

   case "C" : $t = $t + 273.15;  break;

   case "K" : $t = $t;  break;

   case "R" : $t = (5*($t - 491.67) / 9) + 273.15;  break;

   default  : return "";
  }

// -------------------------------
// CONVERT K TO OUTPUT ($ToScale).

   switch (StrToUpper(trim($ToScale)))
  {
   case "F" : return (9 * ($t - 273.15) / 5) + 32;

   case "C" : return $t - 273.15;

   case "K" : return $t;

   case "R" : return (9 * ($t - 273.15) / 5) + 491.67;

   default  : return "";
  }

  } //End of  if (is_numeric)


else


  {
   return "";
  }

} // End of  Temp_To_Temp()

// ****************************************************************************
// ************************************************************************
// ************************************************************************

   function X_ISOymd_For_JD ($JD, $CalMode=1)
{

// ------------------------------------------------------
// COMPUTE JD12 FROM ASTRONOMICAL JD AND THEN COMPUTE THE
// CORRSPONDING ISO INTEGER-ENCODED DATE FOR JD12.

   $a = floor($JD + 0.5);
   $b = $CalMode*floor(($a - 1867216.25) / 36524.25);
   $c = $a + $CalMode*($b - floor($b/4) + 1);
   $e = $c + 1524;
   $f = floor(($e - 122.1) / 365.25);
   $g = floor(365.25 * $f);
   $h = floor(($e - $g) / 30.6001);

   $m = $h - 12*floor($h/14) - 1;
   $d = $e - $g - floor(30.6001 * $h);

   $w = $f - 4716 + floor((14-$m) / 12);
   $y = ($w > 0) ? $w : $w-1;

   $wMinus1 = $w-1;


   $YearSignVal = ($y < 0) ? -1 : 1;

   $ISOymd = $YearSignVal*(10000*abs($y) + 100*$m + $d);

   $Mmm = Month_String($m);

   if ($b == "-0") {$b=0;}

   $dd = sprintf("%02d", $d);

   $out =
"
 JD = $JD
 CalMode = $CalMode

 a = $a = floor($JD + 0.5)
 b = $b = $CalMode*floor(($a - 1867216.25) / 36524.25)
 c = $c = $a + $CalMode*($b - floor($b/4) + 1)
 e = $e = $c + 1524
 f = $f = floor(($e - 122.1) / 365.25)
 g = $g = floor(365.25 * $f)
 h = $h = floor(($e - $g) / 30.6001)

 m = $m = $h - 12*floor($h/14) - 1
 d = $d = $e - $g - floor(30.6001 * $h)

 w = $w = $f - 4716 + floor((14-$m) / 12)
 y = $y = ($w > 0) ? $w:$wMinus1
 YearSignVal = $YearSignVal = ($y < 0) ? -1:1

ISOymd = $ISOymd = $YearSignVal*(10000*abs($y) + 100*$m + $d)
       = $y $Mmm $dd
";

   return "<PRE>$out</PRE>";

} // End of  x_ISOymd_For_JD()



// ************************************************************************
// ************************************************************************




   function x_JD_For_ISOymd ($ISOymd, $CalMode=1)
{

   $BCAD = ($ISOymd < 0)? "B.C." : "A.D.";

   $CalendarMode = ($CalMode == 1) ? "Gregorian" : "Julian";

// ----------------------------------------------
// EXTRACT INDIVIDUAL DATE ARGUMENTS (y,m,d) FROM
// WITHIN ISO INTEGER-ENCODED DATE ARGUMENT.

   $u = abs($ISOymd);
   $v = floor($u / 10000);

   $YearSignVal = ($ISOymd < 0) ? -1 : 1;

   $y = $YearSignVal * $v;
   $m = floor(($u - 10000*$v) / 100);
   $d = $u - 10000*$v - 100*$m;

// -------------------------------------------------
// COMPUTE THE JD NUMBER FOR 12h ON GIVEN DATE. THIS
// NUMBER WILL ALWAYS EQUATE TO AN INTEGER VALUE.

   $w = ($y < 0) ? $y+1 : $y;

   $yPlus1 = $y+1;

   $a = floor((14-$m) / 12);
   $b = $w - $a;
   $c = floor($b/100);

   $Mmm = Month_String($m);

   $e = floor(30.6001*(12*$a + $m + 1));
   $f = floor(365.25*($b + 4716)) - 1524;
   $g = $CalMode*(floor($c/4) - $c + 2) + $d;

   $gLine = str_replace("- -", "+ ", "$CalMode*(floor($c/4) - $c + 2) + $d");

   $dd = sprintf("%02d", $d);

   $z = "$e + $f + $g";
   $z = str_replace("+ -", "- ", $z);

   $h = $e + $f + $g;

   $JD12 = $h;

   $JD00 = $JD12 - 0.5;

   $out =
"
 Date    = $y $Mmm $dd
 CalMode = $CalMode ($CalendarMode)

 -------------------------------------------------
 EXTRACT THE DATE ELEMENTS (y, m, d) FROM (ISOymd)

 YearSignVal = $YearSignVal ($BCAD year) = (ISOymd < 0) ? -1 : 1

 u = $u = abs(ISOymd)
 v = $v = floor(u / 10000)

 y = $y = $YearSignVal * $v
 m = $m = floor(($u - 10000*$v) / 100)
 d = $d = $u - 10000*$v - 100*$m

 --------------------------------------
 ADJUST MATHEMATICAL YEAR (w) FOR BC/AD

 w = $w = ($y < 0) ? $yPlus1 : $y

 --------------------------------------------------------------------------
 JD FOR MIDPOINT OF DATE (12h) = STANDARD CALENDRICAL JD FOR DATE (INTEGER)

 a = $a = floor((14 - $m) / 12)
 b = $b = $w - $a
 c = $c = floor($b/100)

 JD12 = $e = floor(30.6001*(12*$a + $m + 1))
      + $f = floor(365.25*($b + 4716)) - 1524
      + $g = $gLine
      = $JD12

 ------------------------------
 JD FOR BEGINNING OF DATE (00h)

 JD00 = $JD00 = JD12 - 0.5
";

   return "<PRE>$out</PRE>";

} // End of  x_JD_For_ISOymd()



?>


